@@ -22,7 +22,7 @@
 #include <wolfssl/wolfcrypt/settings.h>
 #include "wolfcrypt/test/test.h"
 
-void wolfssl_thread_entry(void* pvParameters)
+void wolfssl_tst_thd_entry(void* pvParameters)
 {
     FSP_PARAMETER_NOT_USED (pvParameters);
     /* Benchmark output is displayed to Renesas Debug Virtual Console */
