@@ -1,12 +1,12 @@
 /* wolfssl_example.c
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -46,6 +46,16 @@
  * Configuration
  ****************************************************************************/
 
+
+#if (!defined(NO_WOLFSSL_CLIENT) || !defined(NO_WOLFSSL_SERVER)) && \
+    !defined(WOLFCRYPT_ONLY) && !defined(SINGLE_THREADED)
+    #define ENABLE_TLS_BENCH
+#endif
+
+#if !defined(WOLFCRYPT_ONLY) && defined(WOLFSSL_TLS13) && !defined(NO_TLS_UART_TEST)
+    #define ENABLE_TLS_UART
+#endif
+
 /* Defaults for configuration parameters */
 #define BENCH_DEFAULT_HOST   "localhost"
 #define BENCH_DEFAULT_PORT   11112
@@ -61,8 +71,8 @@
 #define MEM_BUFFER_SZ        (TEST_PACKET_SIZE + 38 + WC_MAX_DIGEST_SIZE)
 /* make sure memory buffer size is large enough */
 #if MEM_BUFFER_SZ < 2048
-	#undef  MEM_BUFFER_SZ
-	#define MEM_BUFFER_SZ 2048
+    #undef  MEM_BUFFER_SZ
+    #define MEM_BUFFER_SZ 2048
 #endif
 #define SHOW_VERBOSE         0 /* 0=tab del (minimal), 1=info, 2=debug, 3=debug w/wolf logs */
 #ifndef WOLFSSL_CIPHER_LIST_MAX_SIZE
@@ -105,7 +115,8 @@
 #endif /* WOLFSSL_STATIC_MEMORY */
 
 
-/* UART definitions */
+/* This sets which UART to use for the console.  It is something you will have
+ * to configure in STMCubeIDE and then change here. */
 #ifndef HAL_CONSOLE_UART
 #define HAL_CONSOLE_UART huart4
 #endif
@@ -115,23 +126,29 @@ extern UART_HandleTypeDef HAL_CONSOLE_UART;
  * Public types/enumerations/variables
  ****************************************************************************/
 typedef struct func_args {
-	int argc;
-	char** argv;
-	int return_code;
+    int argc;
+    char** argv;
+    int return_code;
 } func_args;
 
 const char menu1[] = "\n"
-		"\tt. WolfCrypt Test\n"
-		"\tb. WolfCrypt Benchmark\n"
-        "\tl. WolfSSL TLS Bench\n"
-        "\te. Show Cipher List\n";
+        "\tt. wolfCrypt Test\n"
+        "\tb. wolfCrypt Benchmark\n"
+    #ifdef ENABLE_TLS_BENCH
+        "\tl. wolfSSL TLS Bench\n"
+    #endif
+        "\te. Show Cipher List\n"
+    #ifdef ENABLE_TLS_UART
+        "\ts. Run TLS 1.3 Server over UART\n"
+        "\tc. Run TLS 1.3 Client over UART\n"
+    #endif
+    ;
 
 static void PrintMemStats(void);
 double current_time(void);
 
 
-#if (!defined(NO_WOLFSSL_CLIENT) || !defined(NO_WOLFSSL_SERVER)) && \
-    !defined(WOLFCRYPT_ONLY) && !defined(SINGLE_THREADED)
+#ifdef ENABLE_TLS_BENCH
 
 static const char* kShutdown = "shutdown";
 
@@ -267,7 +284,7 @@ typedef struct {
 typedef struct {
     int ret;
 
-    osThreadId threadId;
+    osThreadId_t threadId;
 #ifdef CMSIS_OS2_H_
     osSemaphoreId_t mutex;
 #else
@@ -347,8 +364,10 @@ static void PrintTlsStats(stats_t* wcStat, const char* desc, const char* cipher,
            wcStat->connTime * 1000,
            wcStat->connTime * 1000 / wcStat->connCount);
 }
+#endif /* ENABLE_TLS_BENCH */
 
 
+#if defined(ENABLE_TLS_BENCH) || defined(ENABLE_TLS_UART)
 #if defined(KEEP_PEER_CERT) || defined(KEEP_OUR_CERT)
 static const char* client_showx509_msg[] = {
     "issuer",
@@ -423,7 +442,7 @@ static void ShowX509(WOLFSSL_X509* x509, const char* hdr)
     }
 #endif /* OPENSSL_EXTRA */
 }
-#endif
+#endif /* KEEP_PEER_CERT || KEEP_OUR_CERT */
 
 
 static const char* client_showpeer_msg[] = {
@@ -481,14 +500,15 @@ static void ShowPeer(WOLFSSL* ssl)
 
   (void)ssl;
 }
+#endif /* ENABLE_TLS_BENCH || ENABLE_TLS_UART */
 
-
+#ifdef ENABLE_TLS_BENCH
 
 /* server send callback */
 static int ServerMemSend(info_t* info, char* buf, int sz)
 {
 #ifdef CMSIS_OS2_H_
-	osSemaphoreAcquire(info->client.mutex, osWaitForever);
+    osSemaphoreAcquire(info->client.mutex, osWaitForever);
 #else
     osSemaphoreWait(info->client.mutex, osWaitForever);
 #endif
@@ -540,13 +560,13 @@ static int ServerMemRecv(info_t* info, char* buf, int sz)
         osSemaphoreRelease(info->server.mutex);
 #ifdef CMSIS_OS2_H_
         if (osThreadFlagsWait(1, osFlagsWaitAny, RECV_WAIT_TIMEOUT) == osFlagsErrorTimeout) {
-        	printf("Server Recv: Timeout!\n");
-        	return WOLFSSL_CBIO_ERR_TIMEOUT;
+            printf("Server Recv: Timeout!\n");
+            return WOLFSSL_CBIO_ERR_TIMEOUT;
         }
         osSemaphoreAcquire(info->server.mutex, osWaitForever);
 #else
-        if (osSignalWait(1, RECV_WAIT_TIMEOUT) == osEventTimeout) {
-        	printf("Server Recv: Timeout!\n");
+        if (osSignalWait(1, RECV_WAIT_TIMEOUT).status == osEventTimeout) {
+            printf("Server Recv: Timeout!\n");
             return WOLFSSL_CBIO_ERR_TIMEOUT;
         }
         osSemaphoreWait(info->server.mutex, osWaitForever);
@@ -636,14 +656,14 @@ static int ClientMemRecv(info_t* info, char* buf, int sz)
         osSemaphoreRelease(info->client.mutex);
 #ifdef CMSIS_OS2_H_
         if (osThreadFlagsWait(1, osFlagsWaitAny, RECV_WAIT_TIMEOUT) == osFlagsErrorTimeout) {
-        	printf("Client Recv: Timeout!\n");
-        	return WOLFSSL_CBIO_ERR_TIMEOUT;
+            printf("Client Recv: Timeout!\n");
+            return WOLFSSL_CBIO_ERR_TIMEOUT;
         }
         osSemaphoreAcquire(info->client.mutex, osWaitForever);
 #else
-        if (osSignalWait(1, RECV_WAIT_TIMEOUT) == osEventTimeout) {
-        	printf("Client Recv: Timeout!\n");
-        	return WOLFSSL_CBIO_ERR_TIMEOUT;
+        if (osSignalWait(1, RECV_WAIT_TIMEOUT).status == osEventTimeout) {
+            printf("Client Recv: Timeout!\n");
+            return WOLFSSL_CBIO_ERR_TIMEOUT;
         }
         osSemaphoreWait(info->client.mutex, osWaitForever);
 #endif
@@ -718,29 +738,29 @@ static int bench_tls_client(info_t* info)
     /* set up client */
 #ifdef WOLFSSL_TLS13
     if (tls13) {
-	#ifdef WOLFSSL_STATIC_MEMORY
-    	ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfTLSv1_3_client_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfTLSv1_3_client_method_ex,
             gWolfCTXCli, sizeof(gWolfCTXCli), WOLFMEM_GENERAL , 10);
-	#else
+    #else
         cli_ctx = wolfSSL_CTX_new(wolfTLSv1_3_client_method());
-	#endif
+    #endif
     }
 #endif
     if (!tls13) {
 #if !defined(WOLFSSL_TLS13)
-	#ifdef WOLFSSL_STATIC_MEMORY
-    	ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfSSLv23_client_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfSSLv23_client_method_ex,
             gWolfCTXCli, sizeof(gWolfCTXCli), WOLFMEM_GENERAL , 10);
-	#else
+    #else
         cli_ctx = wolfSSL_CTX_new(wolfSSLv23_client_method());
-	#endif
+    #endif
 #elif !defined(WOLFSSL_NO_TLS12)
-	#ifdef WOLFSSL_STATIC_MEMORY
-        ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfTLSv1_2_client_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&cli_ctx, wolfTLSv1_2_client_method_ex,
             gWolfCTXCli, sizeof(gWolfCTXCli), WOLFMEM_GENERAL , 10);
-	#else
+    #else
         cli_ctx = wolfSSL_CTX_new(wolfTLSv1_2_client_method());
-	#endif
+    #endif
 #endif
     }
 
@@ -750,7 +770,7 @@ static int bench_tls_client(info_t* info)
     }
 
 #ifdef WOLFSSL_STATIC_MEMORY
-    ret = wolfSSL_CTX_load_static_memory(&cli_ctx, 0, gWolfIOCli, sizeof(gWolfIOCli), 
+    ret = wolfSSL_CTX_load_static_memory(&cli_ctx, 0, gWolfIOCli, sizeof(gWolfIOCli),
         WOLFMEM_IO_POOL, 10 );
 #endif
 
@@ -929,7 +949,7 @@ static int bench_tls_client(info_t* info)
 exit:
 
     if (ret != 0 && ret != WOLFSSL_SUCCESS) {
-    	info->doShutdown = 1;
+        info->doShutdown = 1;
         printf("Client Error: %d (%s)\n", ret,
             wolfSSL_ERR_reason_error_string(ret));
     }
@@ -966,7 +986,7 @@ static void client_thread(const void* args)
 #ifdef CMSIS_OS2_H_
             osThreadFlagsSet(info->server.threadId, 1);
 #else
-        	osSignalSet(info->server.threadId, 1);
+            osSignalSet(info->server.threadId, 1);
 #endif
         }
         info->client.ret = ret;
@@ -974,7 +994,7 @@ static void client_thread(const void* args)
         osThreadSuspend(info->client.threadId);
 
         if (info->doShutdown)
-            info->client.done = 1;
+            info->server.done = 1;
     } while (!info->doShutdown);
 
     osThreadTerminate(info->client.threadId);
@@ -994,29 +1014,29 @@ static int bench_tls_server(info_t* info)
     /* set up server */
 #ifdef WOLFSSL_TLS13
     if (tls13) {
-	#ifdef WOLFSSL_STATIC_MEMORY
-    	ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfTLSv1_3_server_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfTLSv1_3_server_method_ex,
             gWolfCTXSrv, sizeof(gWolfCTXSrv), WOLFMEM_GENERAL , 10);
-	#else
-    	srv_ctx = wolfSSL_CTX_new(wolfTLSv1_3_server_method());
-	#endif
+    #else
+        srv_ctx = wolfSSL_CTX_new(wolfTLSv1_3_server_method());
+    #endif
     }
 #endif
     if (!tls13) {
 #if !defined(WOLFSSL_TLS13)
-	#ifdef WOLFSSL_STATIC_MEMORY
-    	ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfSSLv23_server_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfSSLv23_server_method_ex,
             gWolfCTXSrv, sizeof(gWolfCTXSrv), WOLFMEM_GENERAL , 10);
-	#else
-    	srv_ctx = wolfSSL_CTX_new(wolfSSLv23_server_method());
-	#endif
+    #else
+        srv_ctx = wolfSSL_CTX_new(wolfSSLv23_server_method());
+    #endif
 #elif !defined(WOLFSSL_NO_TLS12)
-	#ifdef WOLFSSL_STATIC_MEMORY
-        ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfTLSv1_2_server_method_ex, 
+    #ifdef WOLFSSL_STATIC_MEMORY
+        ret = wolfSSL_CTX_load_static_memory(&srv_ctx, wolfTLSv1_2_server_method_ex,
             gWolfCTXSrv, sizeof(gWolfCTXSrv), WOLFMEM_GENERAL , 10);
-	#else
+    #else
         srv_ctx = wolfSSL_CTX_new(wolfTLSv1_2_server_method());
-	#endif
+    #endif
 #endif
     }
 
@@ -1026,7 +1046,7 @@ static int bench_tls_server(info_t* info)
     }
 
 #ifdef WOLFSSL_STATIC_MEMORY
-    ret = wolfSSL_CTX_load_static_memory(&srv_ctx, 0, gWolfIOSrv, sizeof(gWolfIOSrv), 
+    ret = wolfSSL_CTX_load_static_memory(&srv_ctx, 0, gWolfIOSrv, sizeof(gWolfIOSrv),
         WOLFMEM_IO_POOL, 10 );
 #endif
 
@@ -1205,7 +1225,7 @@ static int bench_tls_server(info_t* info)
 exit:
 
     if (ret != 0 && ret != WOLFSSL_SUCCESS) {
-    	info->doShutdown = 1;
+        info->doShutdown = 1;
         printf("Server Error: %d (%s)\n", ret,
             wolfSSL_ERR_reason_error_string(ret));
     }
@@ -1239,7 +1259,7 @@ static void server_thread(const void* args)
         /* signal client */
         if (!info->client.done && info->client.threadId != 0) {
 #ifdef CMSIS_OS2_H_
-        	osThreadFlagsSet(info->client.threadId, 1);
+            osThreadFlagsSet(info->client.threadId, 1);
 #else
             osSignalSet(info->client.threadId, 1);
 #endif
@@ -1249,7 +1269,7 @@ static void server_thread(const void* args)
         osThreadSuspend(info->server.threadId);
 
         if (info->doShutdown)
-            info->server.done = 1;
+            info->client.done = 1;
     } while (!info->doShutdown);
 
     osThreadTerminate(info->server.threadId);
@@ -1441,7 +1461,8 @@ exit:
 
     return ret;
 }
-#endif /* (!NO_WOLFSSL_CLIENT || !NO_WOLFSSL_SERVER) && !WOLFCRYPT_ONLY && !SINGLE_THREADED */
+#endif /* ENABLE_TLS_BENCH */
+
 
 #ifndef WOLFCRYPT_ONLY
 static void ShowCiphers(void)
@@ -1468,25 +1489,325 @@ static void PrintMemStats(void)
 #endif
 }
 
-#if 0
-static void* wolfMallocCb(size_t size)
-{
-	void* ptr = malloc(size);
-	if (ptr == NULL) {
-		printf("BREAK!\n");
-	}
-	return ptr;
+
+#ifdef ENABLE_TLS_UART
+
+/* UART DMA IO Routines */
+#ifndef B115200
+#define B115200 115200
+#endif
+
+/* Max buffer for a single TLS frame */
+#ifndef MAX_RECORD_SIZE
+#define MAX_RECORD_SIZE (16 * 1024)
+#endif
+
+typedef struct {
+    int curr_index;
+    int data_len;
+    char buf[MAX_RECORD_SIZE];
+} tls13_buf;
+
+/* This sets which UART to do the TLS 1.3 connection over.  It is something you
+ * will have to configure in STMCubeIDE and then change here. */
+#ifndef TLS_UART
+#define TLS_UART huart2
+#endif
+/* If you get an undefined error here you can optionally disable the TLS
+ * over UART test using NO_TLS_UART_TEST */
+extern UART_HandleTypeDef TLS_UART;
+
+static int msg_length = 0;
+
+void HAL_UARTEx_RxEventCallback(UART_HandleTypeDef *huart, uint16_t Size) {
+   if (huart->Instance == TLS_UART.Instance) {
+       msg_length = Size;
+   }
 }
-static void wolfFreeCb(void *ptr)
+
+static int uartIORx(WOLFSSL *ssl, char *buf, int sz, void *ctx)
 {
-	free(ptr);
+    HAL_StatusTypeDef status;
+    tls13_buf *tb = ctx;
+
+#ifdef DEBUG_UART_IO
+    printf("UART Read: In %d\n", sz);
+#endif
+
+    if (tb->curr_index + sz <= tb->data_len) {
+        XMEMCPY(buf, tb->buf + tb->curr_index, sz);
+        tb->curr_index += sz;
+#ifdef DEBUG_UART_IO
+        printf("UART Read1: Out %d\n", sz);
+#endif
+        return sz;
+    }
+
+    msg_length = 0;
+    XMEMSET(tb, 0, sizeof(*tb));
+
+    /* Now setup the DMA RX */
+    /* This requires enabling the UART RX DMA in the STM32Cube tool
+     * Under Connectivity click on your TLS UART (USART2) and goto DMA Settings
+     * and Add one for USART2_RX with default options */
+    status = HAL_UARTEx_ReceiveToIdle_DMA(&TLS_UART, (uint8_t *)tb->buf, MAX_RECORD_SIZE);
+    if (status != HAL_OK) {
+        return WOLFSSL_CBIO_ERR_WANT_READ;
+    }
+    else {
+        /* We now go into an infinite loop waiting for msg_length to be set to a
+         * value other than 0. This will be done when the other side writes to
+         * UART and then idles. That will trigger HAL_UARTEx_RxEventCallback()
+         * which will set msg_length to the length of data written.
+         *
+         * If you mistakenly get stuck here, please simply reset the board.
+         */
+        while (msg_length == 0) {
+            HAL_Delay(10);
+        }
+#ifdef DEBUG_UART_IO
+        printf("Message received! length = %d\n", msg_length);
+#endif
+    }
+
+    /* now return the number of bytes requested. */
+    XMEMCPY(buf, tb->buf, sz);
+    tb->data_len = msg_length;
+    tb->curr_index = sz;
+
+#ifdef DEBUG_UART_IO
+    printf("UART Read2: Out %d\n", tb->data_len);
+#endif
+
+    return sz;
 }
-static void* wolfReallocCb(void *ptr, size_t size)
+
+static int uartIOTx(WOLFSSL *ssl, char *buf, int sz, void *ctx)
 {
-	return realloc(ptr, size);
+    HAL_StatusTypeDef status;
+    int ret = sz;
+    (void)ctx;
+
+#ifdef DEBUG_UART_IO
+    printf("UART Write: In %d\n", sz);
+#endif
+
+    status = HAL_UART_Transmit(&TLS_UART, (uint8_t *)buf, sz, 0xFFFF);
+    if (status != HAL_OK) {
+        ret = WOLFSSL_CBIO_ERR_WANT_WRITE;
+    }
+
+#ifdef DEBUG_UART_IO
+    printf("UART Write: Out %d\n", ret);
+#endif
+
+    return ret;
+}
+
+static void uartReset(void)
+{
+    HAL_UART_Abort_IT(&TLS_UART);
+}
+
+/* UART TLS 1.3 client and server */
+#ifndef NO_WOLFSSL_SERVER
+static int tls13_uart_server(void)
+{
+    int ret = -1, err;
+    WOLFSSL_CTX* ctx = NULL;
+    WOLFSSL* ssl = NULL;
+    byte echoBuffer[100];
+#ifdef WOLFSSL_SMALL_STACK
+    tls13_buf* tbuf = (tls13_buf*)XMALLOC(sizeof(*tbuf), NULL,
+                                          DYNAMIC_TYPE_TMP_BUFFER);
+    if (tbuf == NULL) {
+        printf("Memory allocation error\n");
+        goto done;
+    }
+#else
+    tls13_buf tbuf[1];
+#endif
+
+    XMEMSET(tbuf, 0, sizeof(tls13_buf));
+
+    ctx = wolfSSL_CTX_new(wolfTLSv1_3_server_method());
+    if (ctx == NULL) {
+        printf("Error creating WOLFSSL_CTX\n");
+        goto done;
+    }
+
+    /* Register wolfSSL send/recv callbacks */
+    uartReset();
+    wolfSSL_CTX_SetIOSend(ctx, uartIOTx);
+    wolfSSL_CTX_SetIORecv(ctx, uartIORx);
+
+    ret = wolfSSL_CTX_use_PrivateKey_buffer(ctx, ecc_key_der_256,
+        sizeof_ecc_key_der_256, WOLFSSL_FILETYPE_ASN1);
+    if (ret != WOLFSSL_SUCCESS) {
+        printf("error loading server private key\n");
+        goto done;
+    }
+
+    ret = wolfSSL_CTX_use_certificate_buffer(ctx, serv_ecc_der_256,
+        sizeof_serv_ecc_der_256, WOLFSSL_FILETYPE_ASN1);
+    if (ret != WOLFSSL_SUCCESS) {
+        printf("error loading server certificate\n");
+        goto done;
+    }
+
+    ssl = wolfSSL_new(ctx);
+    if (ssl == NULL) {
+        printf("Error creating WOLFSSL\n");
+        goto done;
+    }
+
+    wolfSSL_SetIOReadCtx(ssl, tbuf);
+
+    printf("Waiting for client\n");
+    do {
+        ret = wolfSSL_accept(ssl);
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    if (ret != WOLFSSL_SUCCESS) {
+        printf("TLS accept error %d\n", err);
+        goto done;
+    }
+    printf("TLS Accept handshake done\n");
+
+    /* Waiting for data to echo */
+    XMEMSET(echoBuffer, 0, sizeof(echoBuffer));
+    do {
+        ret = wolfSSL_read(ssl, echoBuffer, sizeof(echoBuffer)-1);
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    printf("Read (%d): %s\n", err, echoBuffer);
+
+    do {
+        ret = wolfSSL_write(ssl, echoBuffer, XSTRLEN((char*)echoBuffer));
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    printf("Sent (%d): %s\n", err, echoBuffer);
+
+    ret = 0; /* Success */
+
+done:
+    if (ssl) {
+        wolfSSL_shutdown(ssl);
+        wolfSSL_free(ssl);
+    }
+    if (ctx) {
+        wolfSSL_CTX_free(ctx);
+    }
+
+#ifdef WOLFSSL_SMALL_STACK
+    XFREE(tbuf, NULL, DYNAMIC_TYPE_TMP_BUFFER);
+#endif
+
+    return ret;
 }
 #endif
 
+#ifdef ENABLE_TLS_UART
+static int tls13_uart_client(void)
+{
+    int ret = -1, err;
+    WOLFSSL_CTX* ctx = NULL;
+    WOLFSSL* ssl = NULL;
+    const char testStr[] = "Testing 1, 2 and 3\r\n";
+    byte readBuf[100];
+#ifdef WOLFSSL_SMALL_STACK
+    tls13_buf* tbuf = (tls13_buf*)XMALLOC(sizeof(*tbuf), NULL,
+                                            DYNAMIC_TYPE_TMP_BUFFER);
+    if (tbuf == NULL) {
+        printf("Memory allocation error\n");
+        goto done;
+    }
+#else
+    tls13_buf tbuf[1];
+#endif
+
+    XMEMSET(tbuf, 0, sizeof(tls13_buf));
+
+    ctx = wolfSSL_CTX_new(wolfTLSv1_3_client_method());
+    if (ctx == NULL) {
+        printf("Error creating WOLFSSL_CTX\n");
+        goto done;
+    }
+
+    /* Register wolfSSL send/recv callbacks */
+    uartReset();
+    wolfSSL_CTX_SetIOSend(ctx, uartIOTx);
+    wolfSSL_CTX_SetIORecv(ctx, uartIORx);
+
+    /* Load the root certificate. */
+    wolfSSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
+
+    ssl = wolfSSL_new(ctx);
+    if (ssl == NULL) {
+        printf("Error creating WOLFSSL\n");
+        goto done;
+    }
+
+    wolfSSL_SetIOReadCtx(ssl, tbuf);
+
+#ifdef WOLFSSL_HAVE_MLKEM
+#ifndef WOLFSSL_NO_ML_KEM
+    if (wolfSSL_UseKeyShare(ssl, WOLFSSL_ML_KEM_512) != WOLFSSL_SUCCESS) {
+        printf("wolfSSL_UseKeyShare Error!!");
+    }
+#else
+    if (wolfSSL_UseKeyShare(ssl, WOLFSSL_KYBER_LEVEL1) != WOLFSSL_SUCCESS) {
+        printf("wolfSSL_UseKeyShare Error!!");
+    }
+#endif
+#endif
+
+    do {
+        ret = wolfSSL_connect(ssl);
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    if (ret != WOLFSSL_SUCCESS) {
+        printf("TLS connect error %d\n", err);
+        goto done;
+    }
+
+    ShowPeer(ssl);
+
+    printf("TLS Connect handshake done\n");
+    printf("Sending test string\n");
+    do {
+        ret = wolfSSL_write(ssl, testStr, XSTRLEN(testStr));
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    printf("Sent (%d): %s\n", err, testStr);
+
+    XMEMSET(readBuf, 0, sizeof(readBuf));
+    do {
+        ret = wolfSSL_read(ssl, readBuf, sizeof(readBuf)-1);
+        err = wolfSSL_get_error(ssl, ret);
+    } while (err == WOLFSSL_ERROR_WANT_READ || err == WOLFSSL_ERROR_WANT_WRITE);
+    printf("Read (%d): %s\n", err, readBuf);
+
+    ret = 0; /* Success */
+
+done:
+    if (ssl) {
+        wolfSSL_shutdown(ssl);
+        wolfSSL_free(ssl);
+    }
+    if (ctx) {
+        wolfSSL_CTX_free(ctx);
+    }
+#ifdef WOLFSSL_SMALL_STACK
+    XFREE(tbuf, NULL, DYNAMIC_TYPE_TMP_BUFFER);
+#endif
+
+    return ret;
+}
+#endif
+#endif /* ENABLE_TLS_UART */
+
+
 /*****************************************************************************
  * Public functions
  ****************************************************************************/
@@ -1494,23 +1815,23 @@ static void* wolfReallocCb(void *ptr, size_t size)
 extern RTC_HandleTypeDef hrtc;
 double current_time(void)
 {
-	RTC_TimeTypeDef time;
-	RTC_DateTypeDef date;
-	uint32_t subsec = 0;
+    RTC_TimeTypeDef time;
+    RTC_DateTypeDef date;
+    uint32_t subsec = 0;
 
-	/* must get time and date here due to STM32 HW bug */
-	HAL_RTC_GetTime(&hrtc, &time, FORMAT_BIN);
-	HAL_RTC_GetDate(&hrtc, &date, FORMAT_BIN);
-	/* Not all STM32 RTCs have subseconds in the struct */
+    /* must get time and date here due to STM32 HW bug */
+    HAL_RTC_GetTime(&hrtc, &time, FORMAT_BIN);
+    HAL_RTC_GetDate(&hrtc, &date, FORMAT_BIN);
+    /* Not all STM32 RTCs have subseconds in the struct */
 #ifdef RTC_ALARMSUBSECONDMASK_ALL
-	subsec = (255 - time.SubSeconds) * 1000 / 255;
+    subsec = (255 - time.SubSeconds) * 1000 / 255;
 #endif
 
-	(void) date;
+    (void) date;
 
-	/* return seconds.milliseconds */
-	return ((double) time.Hours * 24) + ((double) time.Minutes * 60)
-			+ (double) time.Seconds + ((double) subsec / 1000);
+    /* return seconds.milliseconds */
+    return ((double) time.Hours * 3600) + ((double) time.Minutes * 60)
+            + (double) time.Seconds + ((double) subsec / 1000);
 }
 #endif /* HAL_RTC_MODULE_ENABLED */
 
@@ -1521,10 +1842,10 @@ void wolfCryptDemo(const void* argument)
 #endif
 {
     HAL_StatusTypeDef halRet;
-	uint8_t buffer[2];
-	func_args args;
+    uint8_t buffer[2];
+    func_args args;
 
-#ifdef DEBUG_WOLFSSL
+#if 0
     wolfSSL_Debugging_ON();
 #endif
 
@@ -1536,52 +1857,52 @@ void wolfCryptDemo(const void* argument)
 #endif
 
 #ifdef WOLFSSL_STATIC_MEMORY
-    if (wc_LoadStaticMemory(&HEAP_HINT, gWolfMem, sizeof(gWolfMem), 
+    if (wc_LoadStaticMemory(&HEAP_HINT, gWolfMem, sizeof(gWolfMem),
             WOLFMEM_GENERAL, 10) != 0) {
-    	printf("unable to load static memory");
+        printf("unable to load static memory");
     }
 #endif
 
     //wolfSSL_SetAllocators(wolfMallocCb, wolfFreeCb, wolfReallocCb);
 
-	while (1) {
+    while (1) {
         memset(&args, 0, sizeof(args));
         args.return_code = NOT_COMPILED_IN; /* default */
 
-		printf("\n\t\t\t\tMENU\n");
-		printf(menu1);
-		printf("Please select one of the above options:\n");
+        printf("\n\t\t\t\tMENU\n");
+        printf(menu1);
+        printf("Please select one of the above options:\n");
 
         do {
-    		halRet = HAL_UART_Receive(&HAL_CONSOLE_UART, buffer, sizeof(buffer), 100);
+            halRet = HAL_UART_Receive(&HAL_CONSOLE_UART, buffer, sizeof(buffer), 100);
         } while (halRet != HAL_OK || buffer[0] == '\n' || buffer[0] == '\r');
 
-		switch (buffer[0]) {
-		case 't':
-			printf("Running wolfCrypt Tests...\n");
+        switch (buffer[0]) {
+        case 't':
+            printf("Running wolfCrypt Tests...\n");
         #ifndef NO_CRYPT_TEST
-			args.return_code = 0;
-			wolfcrypt_test(&args);
+            args.return_code = 0;
+            wolfcrypt_test(&args);
         #else
             args.return_code = NOT_COMPILED_IN;
         #endif
-			printf("Crypt Test: Return code %d\n", args.return_code);
-			break;
+            printf("Crypt Test: Return code %d\n", args.return_code);
+            break;
 
-		case 'b':
-			printf("Running wolfCrypt Benchmarks...\n");
+        case 'b':
+            printf("Running wolfCrypt Benchmarks...\n");
         #ifndef NO_CRYPT_BENCHMARK
-			args.return_code = 0;
-			benchmark_test(&args);
+            args.return_code = 0;
+            benchmark_test(&args);
         #else
             args.return_code = NOT_COMPILED_IN;
         #endif
-			printf("Benchmark Test: Return code %d\n", args.return_code);
-			break;
+            printf("Benchmark Test: Return code %d\n", args.return_code);
+            break;
 
         case 'l':
             printf("Running TLS Benchmarks...\n");
-        #if (!defined(NO_WOLFSSL_CLIENT) || !defined(NO_WOLFSSL_SERVER)) && !defined(WOLFCRYPT_ONLY) && !defined(SINGLE_THREADED)
+        #ifdef ENABLE_TLS_BENCH
             bench_tls(&args);
         #else
             args.return_code = NOT_COMPILED_IN;
@@ -1596,15 +1917,35 @@ void wolfCryptDemo(const void* argument)
             printf("Not compiled in\n");
         #endif
             break;
+#ifdef ENABLE_TLS_UART
+        case 's':
+        #ifndef NO_WOLFSSL_SERVER
+            printf("Running TLS 1.3 server...\n");
+            args.return_code = tls13_uart_server();
+        #else
+            args.return_code = NOT_COMPILED_IN;
+        #endif
+            printf("TLS 1.3 Server: Return code %d\n", args.return_code);
+            break;
 
-			// All other cases go here
-		default:
-			printf("\nSelection out of range\n");
-			break;
-		}
+        case 'c':
+        #ifndef NO_WOLFSSL_CLIENT
+            printf("Running TLS 1.3 client...\n");
+            args.return_code = tls13_uart_client();
+        #else
+            args.return_code = NOT_COMPILED_IN;
+        #endif
+            printf("TLS 1.3 Client: Return code %d\n", args.return_code);
+            break;
+#endif /* ENABLE_TLS_UART */
+            /* All other cases go here */
+        default:
+            printf("\nSelection out of range\n");
+            break;
+        }
 
         PrintMemStats();
-	}
+    }
 
 #ifdef WOLFCRYPT_ONLY
     wolfCrypt_Cleanup();
