@@ -41,7 +41,7 @@
     \code
     ecc_key key;
     wc_ecc_init(&key);
-    WC_WC_RNG rng;
+    WC_RNG rng;
     wc_InitRng(&rng);
     wc_ecc_make_key(&rng, 32, &key); // initialize 32 byte ecc key
     \endcode
@@ -49,7 +49,7 @@
     \sa wc_ecc_init
     \sa wc_ecc_shared_secret
 */
-WOLFSSL_API
+
 int wc_ecc_make_key(WC_RNG* rng, int keysize, ecc_key* key);
 
 /*!
@@ -95,7 +95,7 @@ int wc_ecc_make_key(WC_RNG* rng, int keysize, ecc_key* key);
     \code
     ecc_key key;
     int ret;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     wc_ecc_init(&key);
     wc_InitRng(&rng);
     int curveId = ECC_SECP521R1;
@@ -110,9 +110,116 @@ int wc_ecc_make_key(WC_RNG* rng, int keysize, ecc_key* key);
     \sa wc_ecc_make_key
     \sa wc_ecc_get_curve_size_from_id
 */
-WOLFSSL_API
+
 int wc_ecc_make_key_ex(WC_RNG* rng, int keysize, ecc_key* key, int curve_id);
 
+/*!
+    \ingroup ECC
+
+    \brief wc_ecc_make_pub computes the public component from an ecc_key with an
+    existing private component.  If pubOut is supplied, the computed public key
+    is stored there, else it is stored in the supplied ecc_key public component
+    slot.
+
+    \return 0 Returned on success.
+    \return ECC_BAD_ARG_E Returned if key is NULL
+    \return BAD_FUNC_ARG Returned if the supplied key is not a valid ecc_key.
+    \return MEMORY_E Returned if there is an error allocating memory while
+    computing the public key
+    \return MP_INIT_E may be returned if there is an error while computing
+    the public key
+    \return MP_READ_E may be returned if there is an error while computing
+    the public key
+    \return MP_CMP_E may be returned if there is an error while computing the
+    public key
+    \return MP_INVMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_EXPTMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_MOD_E may be returned if there is an error while computing the
+    public key
+    \return MP_MUL_E may be returned if there is an error while computing the
+    public key
+    \return MP_ADD_E may be returned if there is an error while computing the
+    public key
+    \return MP_MULMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_TO_E may be returned if there is an error while computing the
+    public key
+    \return MP_MEM may be returned if there is an error while computing the
+    public key
+    \return ECC_OUT_OF_RANGE_E may be returned if there is an error while computing the
+    public key
+    \return ECC_PRIV_KEY_E may be returned if there is an error while computing the
+    public key
+    \return ECC_INF_E may be returned if there is an error while computing the
+    public key
+
+    \param key Pointer to an ecc_key containing a valid private component
+    \param pubOut Optional pointer to an ecc_point struct in which to store
+    the computed public key
+
+    \sa wc_ecc_make_pub_ex
+    \sa wc_ecc_make_key
+*/
+
+int wc_ecc_make_pub(ecc_key* key, ecc_point* pubOut);
+
+/*!
+    \ingroup ECC
+
+    \brief wc_ecc_make_pub_ex computes the public component from an ecc_key with
+    an existing private component.  If pubOut is supplied, the computed public
+    key is stored there, else it is stored in the supplied ecc_key public
+    component slot.  The supplied rng, if non-NULL, is used to blind the private
+    key value used in the computation.
+
+    \return 0 Returned on success.
+    \return ECC_BAD_ARG_E Returned if key is NULL
+    \return BAD_FUNC_ARG Returned if the supplied key is not a valid ecc_key.
+    \return MEMORY_E Returned if there is an error allocating memory while
+    computing the public key
+    \return MP_INIT_E may be returned if there is an error while computing
+    the public key
+    \return MP_READ_E may be returned if there is an error while computing
+    the public key
+    \return MP_CMP_E may be returned if there is an error while computing the
+    public key
+    \return MP_INVMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_EXPTMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_MOD_E may be returned if there is an error while computing the
+    public key
+    \return MP_MUL_E may be returned if there is an error while computing the
+    public key
+    \return MP_ADD_E may be returned if there is an error while computing the
+    public key
+    \return MP_MULMOD_E may be returned if there is an error while computing
+    the public key
+    \return MP_TO_E may be returned if there is an error while computing the
+    public key
+    \return MP_MEM may be returned if there is an error while computing the
+    public key
+    \return ECC_OUT_OF_RANGE_E may be returned if there is an error while computing the
+    public key
+    \return ECC_PRIV_KEY_E may be returned if there is an error while computing the
+    public key
+    \return ECC_INF_E may be returned if there is an error while computing the
+    public key
+
+    \param key Pointer to an ecc_key containing a valid private component
+    \param pubOut Optional pointer to an ecc_point struct in which to store
+    the computed public key
+    \param rng Rng to be used in the public key computation
+
+    \sa wc_ecc_make_pub
+    \sa wc_ecc_make_key
+    \sa wc_InitRng
+*/
+
+int wc_ecc_make_pub_ex(ecc_key* key, ecc_point* pubOut, WC_RNG* rng);
+
 /*!
     \ingroup ECC
 
@@ -127,7 +234,7 @@ int wc_ecc_make_key_ex(WC_RNG* rng, int keysize, ecc_key* key, int curve_id);
     _Example_
     \code
     ecc_key key;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     int check_result;
     wc_ecc_init(&key);
     wc_InitRng(&rng);
@@ -146,7 +253,7 @@ int wc_ecc_make_key_ex(WC_RNG* rng, int keysize, ecc_key* key, int curve_id);
 
     \sa wc_ecc_point_is_at_infinity
 */
-WOLFSSL_API
+
 int wc_ecc_check_key(ecc_key* key);
 
 /*!
@@ -167,7 +274,7 @@ int wc_ecc_check_key(ecc_key* key);
     \sa wc_ecc_key_new
     \sa wc_ecc_init_ex
 */
-WOLFSSL_API
+
 void wc_ecc_key_free(ecc_key* key);
 
 /*!
@@ -223,7 +330,7 @@ void wc_ecc_key_free(ecc_key* key);
     _Example_
     \code
     ecc_key priv, pub;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     byte secret[1024]; // can hold 1024 byte shared secret key
     word32 secretSz = sizeof(secret);
     int ret;
@@ -242,7 +349,7 @@ void wc_ecc_key_free(ecc_key* key);
     \sa wc_ecc_init
     \sa wc_ecc_make_key
 */
-WOLFSSL_API
+
 int wc_ecc_shared_secret(ecc_key* private_key, ecc_key* public_key, byte* out,
                       word32* outlen);
 
@@ -288,7 +395,7 @@ int wc_ecc_shared_secret(ecc_key* private_key, ecc_key* public_key, byte* out,
 
     \sa wc_ecc_verify_hash_ex
 */
-WOLFSSL_API
+
 int wc_ecc_shared_secret_ex(ecc_key* private_key, ecc_point* point,
                              byte* out, word32 *outlen);
 
@@ -340,7 +447,7 @@ int wc_ecc_shared_secret_ex(ecc_key* private_key, ecc_point* point,
     _Example_
     \code
     ecc_key key;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     int ret, sigSz;
 
     byte sig[512]; // will hold generated signature
@@ -357,7 +464,7 @@ int wc_ecc_shared_secret_ex(ecc_key* private_key, ecc_point* point,
 
     \sa wc_ecc_verify_hash
 */
-WOLFSSL_API
+
 int wc_ecc_sign_hash(const byte* in, word32 inlen, byte* out, word32 *outlen,
                      WC_RNG* rng, ecc_key* key);
 
@@ -406,7 +513,7 @@ int wc_ecc_sign_hash(const byte* in, word32 inlen, byte* out, word32 *outlen,
     _Example_
     \code
     ecc_key key;
-    WC_WC_WC_RNG rng;
+    WC_RNG rng;
     int ret, sigSz;
     mp_int r; // destination for r component of signature.
     mp_int s; // destination for s component of signature.
@@ -428,7 +535,7 @@ int wc_ecc_sign_hash(const byte* in, word32 inlen, byte* out, word32 *outlen,
 
     \sa wc_ecc_verify_hash_ex
 */
-WOLFSSL_API
+
 int wc_ecc_sign_hash_ex(const byte* in, word32 inlen, WC_RNG* rng,
                         ecc_key* key, mp_int *r, mp_int *s);
 
@@ -496,7 +603,7 @@ int wc_ecc_sign_hash_ex(const byte* in, word32 inlen, WC_RNG* rng,
     \sa wc_ecc_sign_hash
     \sa wc_ecc_verify_hash_ex
 */
-WOLFSSL_API
+
 int wc_ecc_verify_hash(const byte* sig, word32 siglen, const byte* hash,
                     word32 hashlen, int* stat, ecc_key* key);
 
@@ -535,7 +642,7 @@ int wc_ecc_verify_hash(const byte* sig, word32 siglen, const byte* hash,
 
     \sa wc_ecc_verify_hash
 */
-WOLFSSL_API
+
 int wc_ecc_verify_hash_ex(mp_int *r, mp_int *s, const byte* hash,
                           word32 hashlen, int* stat, ecc_key* key);
 
@@ -559,7 +666,7 @@ int wc_ecc_verify_hash_ex(mp_int *r, mp_int *s, const byte* hash,
     \sa wc_ecc_make_key
     \sa wc_ecc_free
 */
-WOLFSSL_API
+
 int wc_ecc_init(ecc_key* key);
 
 /*!
@@ -572,8 +679,8 @@ int wc_ecc_init(ecc_key* key);
     \return MEMORY_E Returned if there is an error allocating memory
 
     \param key pointer to the ecc_key object to initialize
-    \param devId ID to use with async hardware
     \param heap pointer to a heap identifier
+    \param devId ID to use with crypto callbacks or async hardware. Set to INVALID_DEVID (-2) if not used
 
     _Example_
     \code
@@ -585,7 +692,7 @@ int wc_ecc_init(ecc_key* key);
     \sa wc_ecc_free
     \sa wc_ecc_init
 */
-WOLFSSL_API
+
 int wc_ecc_init_ex(ecc_key* key, void* heap, int devId);
 
 /*!
@@ -607,7 +714,7 @@ int wc_ecc_init_ex(ecc_key* key, void* heap, int devId);
     \sa wc_ecc_key_free
     \sa wc_ecc_init
 */
-WOLFSSL_API
+
 ecc_key* wc_ecc_key_new(void* heap);
 
 /*!
@@ -628,7 +735,7 @@ ecc_key* wc_ecc_key_new(void* heap);
 
     \sa wc_ecc_init
 */
-WOLFSSL_API
+
 int wc_ecc_free(ecc_key* key);
 
 /*!
@@ -636,7 +743,8 @@ int wc_ecc_free(ecc_key* key);
 
     \brief This function frees the fixed-point cache, which can be used
     with ecc to speed up computation times. To use this functionality,
-    FP_ECC (fixed-point ecc), should be defined.
+    FP_ECC (fixed-point ecc), should be defined. Threaded applications should
+    call this function before exiting the thread.
 
     \return none No returns.
 
@@ -653,7 +761,7 @@ int wc_ecc_free(ecc_key* key);
 
     \sa wc_ecc_free
 */
-WOLFSSL_API
+
 void wc_ecc_fp_free(void);
 
 /*!
@@ -669,7 +777,7 @@ void wc_ecc_fp_free(void);
     _Example_
     \code
     ecc_key key;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     int is_valid;
     wc_ecc_init(&key);
     wc_InitRng(&rng);
@@ -687,7 +795,7 @@ void wc_ecc_fp_free(void);
 
     \sa none
 */
-WOLFSSL_API
+
 int wc_ecc_is_valid_idx(int n);
 
 /*!
@@ -715,7 +823,7 @@ int wc_ecc_is_valid_idx(int n);
     \sa wc_ecc_cmp_point
     \sa wc_ecc_copy_point
 */
-WOLFSSL_API
+
 ecc_point* wc_ecc_new_point(void);
 
 /*!
@@ -743,7 +851,7 @@ ecc_point* wc_ecc_new_point(void);
     \sa wc_ecc_cmp_point
     \sa wc_ecc_copy_point
 */
-WOLFSSL_API
+
 void wc_ecc_del_point(ecc_point* p);
 
 /*!
@@ -776,7 +884,7 @@ void wc_ecc_del_point(ecc_point* p);
     \sa wc_ecc_cmp_point
     \sa wc_ecc_del_point
 */
-WOLFSSL_API
+
 int wc_ecc_copy_point(ecc_point* p, ecc_point *r);
 
 /*!
@@ -819,7 +927,7 @@ int wc_ecc_copy_point(ecc_point* p, ecc_point *r);
     \sa wc_ecc_del_point
     \sa wc_ecc_copy_point
 */
-WOLFSSL_API
+
 int wc_ecc_cmp_point(ecc_point* a, ecc_point *b);
 
 /*!
@@ -860,7 +968,7 @@ int wc_ecc_cmp_point(ecc_point* a, ecc_point *b);
     \sa wc_ecc_cmp_point
     \sa wc_ecc_copy_point
 */
-WOLFSSL_API
+
 int wc_ecc_point_is_at_infinity(ecc_point *p);
 
 /*!
@@ -894,7 +1002,7 @@ int wc_ecc_point_is_at_infinity(ecc_point *p);
 
     \sa none
 */
-WOLFSSL_API
+
 int wc_ecc_mulmod(mp_int* k, ecc_point *G, ecc_point *R,
                   mp_int* a, mp_int* modulus, int map);
 
@@ -959,30 +1067,33 @@ int wc_ecc_mulmod(mp_int* k, ecc_point *G, ecc_point *R,
 
     \sa wc_ecc_export_x963_ex
     \sa wc_ecc_import_x963
+    \sa wc_ecc_make_pub
 */
-WOLFSSL_API
-int wc_ecc_export_x963(ecc_key*, byte* out, word32* outLen);
+
+int wc_ecc_export_x963(ecc_key* key, byte* out, word32* outLen);
 
 /*!
     \ingroup ECC
 
-    \brief This function exports the ECC key from the ecc_key structure,
+    \brief This function exports the public key from the ecc_key structure,
     storing the result in out. The key will be stored in ANSI X9.63 format.
     It stores the bytes written to the output buffer in outLen. This function
     allows the additional option of compressing the certificate through the
     compressed parameter. When this parameter is true, the key will be stored
     in ANSI X9.63 compressed format.
 
-    \return 0 Returned on successfully exporting the ecc_key
+    \return 0 Returned on successfully exporting the ecc_key public component
+    \return ECC_PRIVATEKEY_ONLY Returned if the ecc_key public component is
+    missing
     \return NOT_COMPILED_IN Returned if the HAVE_COMP_KEY was not enabled at
     compile time, but the key was requested in compressed format
     \return LENGTH_ONLY_E Returned if the output buffer evaluates to NULL, but
     the other two input parameters are valid. Indicates that the function is
-    only returning the length required to store the key
+    only returning the length required to store the public key
     \return ECC_BAD_ARG_E Returned if any of the input parameters are NULL, or
     the key is unsupported (has an invalid index)
     \return BUFFER_E Returned if the output buffer is too small to store the
-    ecc key. If the output buffer is too small, the size needed will be
+    public key. If the output buffer is too small, the size needed will be
     returned in outLen
     \return MEMORY_E Returned if there is an error allocating memory with
     XMALLOC
@@ -1009,9 +1120,9 @@ int wc_ecc_export_x963(ecc_key*, byte* out, word32* outLen);
 
     \param key pointer to the ecc_key object to export
     \param out pointer to the buffer in which to store the ANSI X9.63
-    formatted key
+    formatted public key
     \param outLen size of the output buffer. On successfully storing the
-    key, will hold the bytes written to the output buffer
+    public key, will hold the bytes written to the output buffer
     \param compressed indicator of whether to store the key in compressed
     format. 1==compressed, 0==uncompressed
 
@@ -1030,9 +1141,10 @@ int wc_ecc_export_x963(ecc_key*, byte* out, word32* outLen);
 
     \sa wc_ecc_export_x963
     \sa wc_ecc_import_x963
+    \sa wc_ecc_make_pub
 */
-WOLFSSL_API
-int wc_ecc_export_x963_ex(ecc_key*, byte* out, word32* outLen, int compressed);
+
+int wc_ecc_export_x963_ex(ecc_key* key, byte* out, word32* outLen, int compressed);
 
 /*!
     \ingroup ECC
@@ -1094,7 +1206,7 @@ int wc_ecc_export_x963_ex(ecc_key*, byte* out, word32* outLen, int compressed);
     \sa wc_ecc_export_x963
     \sa wc_ecc_import_private_key
 */
-WOLFSSL_API
+
 int wc_ecc_import_x963(const byte* in, word32 inLen, ecc_key* key);
 
 /*!
@@ -1163,7 +1275,7 @@ int wc_ecc_import_x963(const byte* in, word32 inLen, ecc_key* key);
     \sa wc_ecc_export_x963
     \sa wc_ecc_import_private_key
 */
-WOLFSSL_API
+
 int wc_ecc_import_private_key(const byte* priv, word32 privSz, const byte* pub,
                            word32 pubSz, ecc_key* key);
 
@@ -1229,7 +1341,7 @@ int wc_ecc_import_private_key(const byte* priv, word32 privSz, const byte* pub,
     \sa wc_ecc_sign_hash
     \sa wc_ecc_sig_size
 */
-WOLFSSL_API
+
 int wc_ecc_rs_to_sig(const char* r, const char* s, byte* out, word32* outlen);
 
 /*!
@@ -1294,7 +1406,7 @@ int wc_ecc_rs_to_sig(const char* r, const char* s, byte* out, word32* outlen);
 
     \sa wc_ecc_import_private_key
 */
-WOLFSSL_API
+
 int wc_ecc_import_raw(ecc_key* key, const char* qx, const char* qy,
                    const char* d, const char* curveName);
 
@@ -1357,7 +1469,7 @@ int wc_ecc_import_raw(ecc_key* key, const char* qx, const char* qy,
 
     \sa wc_ecc_import_private_key
 */
-WOLFSSL_API
+
 int wc_ecc_export_private_only(ecc_key* key, byte* out, word32* outLen);
 
 /*!
@@ -1389,7 +1501,7 @@ int wc_ecc_export_private_only(ecc_key* key, byte* out, word32* outLen);
 
     \sa wc_ecc_import_point_der
 */
-WOLFSSL_API
+
 int wc_ecc_export_point_der(const int curve_idx, ecc_point* point,
                             byte* out, word32* outLen);
 
@@ -1421,7 +1533,7 @@ int wc_ecc_export_point_der(const int curve_idx, ecc_point* point,
 
     \sa wc_ecc_export_point_der
 */
-WOLFSSL_API
+
 int wc_ecc_import_point_der(byte* in, word32 inLen, const int curve_idx,
                             ecc_point* point);
 
@@ -1449,7 +1561,7 @@ int wc_ecc_import_point_der(byte* in, word32 inLen, const int curve_idx,
 
     \sa wc_ecc_make_key
 */
-WOLFSSL_API
+
 int wc_ecc_size(ecc_key* key);
 
 /*!
@@ -1475,7 +1587,7 @@ int wc_ecc_size(ecc_key* key);
     \sa wc_ecc_sign_hash
     \sa wc_ecc_sig_size
 */
-WOLFSSL_API
+
 int wc_ecc_sig_size_calc(int sz);
 
 
@@ -1508,7 +1620,7 @@ int wc_ecc_sig_size_calc(int sz);
     \sa wc_ecc_sign_hash
     \sa wc_ecc_sig_size_calc
 */
-WOLFSSL_API
+
 int wc_ecc_sig_size(ecc_key* key);
 
 
@@ -1530,7 +1642,7 @@ int wc_ecc_sig_size(ecc_key* key);
     _Example_
     \code
     ecEncCtx* ctx;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     wc_InitRng(&rng);
     ctx = wc_ecc_ctx_new(REQ_RESP_CLIENT, &rng);
     if(ctx == NULL) {
@@ -1542,7 +1654,7 @@ int wc_ecc_sig_size(ecc_key* key);
     \sa wc_ecc_encrypt_ex
     \sa wc_ecc_decrypt
 */
-WOLFSSL_API
+
 ecEncCtx* wc_ecc_ctx_new(int flags, WC_RNG* rng);
 
 /*!
@@ -1558,7 +1670,7 @@ ecEncCtx* wc_ecc_ctx_new(int flags, WC_RNG* rng);
     _Example_
     \code
     ecEncCtx* ctx;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     wc_InitRng(&rng);
     ctx = wc_ecc_ctx_new(REQ_RESP_CLIENT, &rng);
     // do secure communication
@@ -1568,7 +1680,7 @@ ecEncCtx* wc_ecc_ctx_new(int flags, WC_RNG* rng);
 
     \sa wc_ecc_ctx_new
 */
-WOLFSSL_API
+
 void wc_ecc_ctx_free(ecEncCtx*);
 
 /*!
@@ -1588,7 +1700,7 @@ void wc_ecc_ctx_free(ecEncCtx*);
     _Example_
     \code
     ecEncCtx* ctx;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     wc_InitRng(&rng);
     ctx = wc_ecc_ctx_new(REQ_RESP_CLIENT, &rng);
     // do secure communication
@@ -1599,8 +1711,8 @@ void wc_ecc_ctx_free(ecEncCtx*);
 
     \sa wc_ecc_ctx_new
 */
-WOLFSSL_API
-int wc_ecc_ctx_reset(ecEncCtx*, WC_RNG*);  /* reset for use again w/o alloc/free */
+
+int wc_ecc_ctx_reset(ecEncCtx* ctx, WC_RNG* rng);  /* reset for use again w/o alloc/free */
 
 /*!
     \ingroup ECC
@@ -1630,7 +1742,7 @@ int wc_ecc_ctx_reset(ecEncCtx*, WC_RNG*);  /* reset for use again w/o alloc/free
 
     \sa wc_ecc_ctx_new
 */
-WOLFSSL_API
+
 int wc_ecc_ctx_set_algo(ecEncCtx* ctx, byte encAlgo, byte kdfAlgo,
     byte macAlgo);
 
@@ -1652,7 +1764,7 @@ int wc_ecc_ctx_set_algo(ecEncCtx* ctx, byte encAlgo, byte kdfAlgo,
     _Example_
     \code
     ecEncCtx* ctx;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     const byte* salt;
     wc_InitRng(&rng);
     ctx = wc_ecc_ctx_new(REQ_RESP_CLIENT, &rng);
@@ -1664,8 +1776,9 @@ int wc_ecc_ctx_set_algo(ecEncCtx* ctx, byte encAlgo, byte kdfAlgo,
 
     \sa wc_ecc_ctx_new
     \sa wc_ecc_ctx_set_peer_salt
+    \sa wc_ecc_ctx_set_kdf_salt
 */
-WOLFSSL_API
+
 const byte* wc_ecc_ctx_get_own_salt(ecEncCtx*);
 
 /*!
@@ -1688,7 +1801,7 @@ const byte* wc_ecc_ctx_get_own_salt(ecEncCtx*);
     _Example_
     \code
     ecEncCtx* cliCtx, srvCtx;
-    WC_WC_RNG rng;
+    WC_RNG rng;
     const byte* cliSalt, srvSalt;
     int ret;
 
@@ -1702,9 +1815,45 @@ const byte* wc_ecc_ctx_get_own_salt(ecEncCtx*);
     \endcode
 
     \sa wc_ecc_ctx_get_own_salt
+    \sa wc_ecc_ctx_set_kdf_salt
 */
-WOLFSSL_API
-int wc_ecc_ctx_set_peer_salt(ecEncCtx*, const byte* salt);
+
+int wc_ecc_ctx_set_peer_salt(ecEncCtx* ctx, const byte* salt);
+
+/*!
+    \ingroup ECC
+
+    \brief This function sets the salt pointer and length to use with KDF
+    into the ecEncCtx object.
+
+    \return 0 Returned upon successfully setting the salt for the
+    ecEncCtx object.
+    \return BAD_FUNC_ARG Returned if the given ecEncCtx object is NULL
+    or if the given salt is NULL and length is not NULL.
+
+    \param ctx pointer to the ecEncCtx for which to set the salt
+    \param salt pointer to salt buffer
+    \param sz length salt in bytes
+
+    _Example_
+    \code
+    ecEncCtx* srvCtx;
+    WC_RNG rng;
+    byte cliSalt[] = { fixed salt data };
+    word32 cliSaltLen = (word32)sizeof(cliSalt);
+    int ret;
+
+    wc_InitRng(&rng);
+    cliCtx = wc_ecc_ctx_new(REQ_RESP_SERVER, &rng);
+
+    ret = wc_ecc_ctx_set_kdf_salt(&cliCtx, cliSalt, cliSaltLen);
+    \endcode
+
+    \sa wc_ecc_ctx_get_own_salt
+    \sa wc_ecc_ctx_get_peer_salt
+*/
+
+int wc_ecc_ctx_set_kdf_salt(ecEncCtx* ctx, const byte* salt, word32 sz);
 
 /*!
     \ingroup ECC
@@ -1734,8 +1883,8 @@ int wc_ecc_ctx_set_peer_salt(ecEncCtx*, const byte* salt);
 
     \sa wc_ecc_ctx_new
 */
-WOLFSSL_API
-int wc_ecc_ctx_set_info(ecEncCtx*, const byte* info, int sz);
+
+int wc_ecc_ctx_set_info(ecEncCtx* ctx, const byte* info, int sz);
 
 /*!
     \ingroup ECC
@@ -1795,7 +1944,7 @@ int wc_ecc_ctx_set_info(ecEncCtx*, const byte* info, int sz);
     \sa wc_ecc_encrypt_ex
     \sa wc_ecc_decrypt
 */
-WOLFSSL_API
+
 int wc_ecc_encrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
                 word32 msgSz, byte* out, word32* outSz, ecEncCtx* ctx);
 
@@ -1859,7 +2008,7 @@ int wc_ecc_encrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
     \sa wc_ecc_encrypt
     \sa wc_ecc_decrypt
 */
-WOLFSSL_API
+
 int wc_ecc_encrypt_ex(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
     word32 msgSz, byte* out, word32* outSz, ecEncCtx* ctx, int compressed);
 
@@ -1922,7 +2071,7 @@ int wc_ecc_encrypt_ex(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
     \sa wc_ecc_encrypt
     \sa wc_ecc_encrypt_ex
 */
-WOLFSSL_API
+
 int wc_ecc_decrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
                 word32 msgSz, byte* out, word32* outSz, ecEncCtx* ctx);
 
@@ -1930,7 +2079,7 @@ int wc_ecc_decrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
 /*!
     \ingroup ECC
 
-    \brief Enable ECC support for non-blocking operations. Supported for 
+    \brief Enable ECC support for non-blocking operations. Supported for
         Single Precision (SP) math with the following build options:
             WOLFSSL_SP_NONBLOCK
             WOLFSSL_SP_SMALL
@@ -1940,7 +2089,7 @@ int wc_ecc_decrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
     \return 0 Returned upon successfully setting the callback context the input message
 
     \param key pointer to the ecc_key object
-    \param ctx pointer to ecc_nb_ctx_t structure with stack data cache for SP 
+    \param ctx pointer to ecc_nb_ctx_t structure with stack data cache for SP
 
     _Example_
     \code
@@ -1960,11 +2109,37 @@ int wc_ecc_decrypt(ecc_key* privKey, ecc_key* pubKey, const byte* msg,
                     &key
                 );
 
-                // TODO: Real-time work can be called here 
+                // TODO: Real-time work can be called here
             } while (ret == FP_WOULDBLOCK);
         }
         wc_ecc_free(&key);
     }
     \endcode
 */
-WOLFSSL_API int wc_ecc_set_nonblock(ecc_key *key, ecc_nb_ctx_t* ctx);
+int wc_ecc_set_nonblock(ecc_key *key, ecc_nb_ctx_t* ctx);
+
+/*!
+    \ingroup ECC
+
+    \brief Compare a curve which has larger key than specified size or the curve matched curve ID,
+         set a curve with smaller key size to the key.
+
+    \return 0 Returned upon successfully setting the key
+
+    \param keysize Key size in bytes
+    \param curve_id Curve ID
+
+                                                                                                        _Example_
+    \code int ret;
+    ecc_key ecc;
+
+    ret = wc_ecc_init(&ecc);
+    if (ret != 0)
+        return ret;
+        ret = wc_ecc_set_curve(&ecc, 32, ECC_SECP256R1));
+        if (ret != 0)
+            return ret;
+
+    \endcode
+*/
+int wc_ecc_set_curve(ecc_key *key, int keysize, int curve_id);
