@@ -1,21 +1,21 @@
 /*!
     \ingroup ED25519
 
-    \brief This function generates the Ed25519 public key from the private key.
-    It stores the public key in the buffer pubKey, and sets the bytes
-    written to this buffer in pubKeySz.
+    \brief This function generates the Ed25519 public key from the private key,
+    stored in the ed25519_key object. It stores the public key in the buffer
+    pubKey.
 
     \return 0 Returned upon successfully making the public key.
-    \return BAD_FUNC_ARG Returned ifi key or pubKey evaluate to NULL, or if the
+    \return BAD_FUNC_ARG Returned if key or pubKey evaluate to NULL, or if the
     specified key size is not 32 bytes (Ed25519 has 32 byte keys).
+    \return ECC_PRIV_KEY_E returned if the ed25519_key object does not have
+    the private key in it.
     \return MEMORY_E Returned if there is an error allocating memory
     during function execution.
 
     \param [in] key Pointer to the ed25519_key for which to generate a key.
-    \param [out] out Pointer to the buffer in which to store the public key.
-    \param [in,out] outLen Pointer to a word32 object with the size available
-    in out. Set with the number of bytes written to out after successfully
-    exporting the public key.
+    \param [out] pubKey Pointer to the buffer in which to store the public key.
+    \param [in] pubKeySz Size of the public key. Should be ED25519_PUB_KEY_SIZE.
 
     _Example_
     \code
@@ -188,8 +188,7 @@ int wc_ed25519ctx_sign_msg(const byte* in, word32 inlen, byte* out,
 
     \brief This function signs a message digest using an ed25519_key object
     to guarantee authenticity. The context is included as part of the data
-    signed. The message is pre-hashed before signature calculation. The hash
-    algorithm used to create message digest must be SHAKE-256.
+    signed. The message is pre-hashed before signature calculation.
 
     \return 0 Returned upon successfully generating a signature for the
     message digest.
@@ -301,7 +300,7 @@ int wc_ed25519ph_sign_msg(const byte* in, word32 inlen, byte* out,
     \ingroup ED25519
 
     \brief This function verifies the Ed25519 signature of a message to ensure
-    authenticity. It returns the answer through res, with 1 corresponding to
+    authenticity. It returns the answer through ret, with 1 corresponding to
     a valid signature, and 0 corresponding to an invalid signature.
 
     \return 0 Returned upon successfully performing the signature
@@ -315,7 +314,7 @@ int wc_ed25519ph_sign_msg(const byte* in, word32 inlen, byte* out,
     \param [in] siglen Length of the signature to verify.
     \param [in] msg Pointer to the buffer containing the message to verify.
     \param [in] msgLen Length of the message to verify.
-    \param [out] res Pointer to the result of the verification. 1 indicates the
+    \param [out] ret Pointer to the result of the verification. 1 indicates the
     message was successfully verified.
     \param [in] key Pointer to a public Ed25519 key with which to verify the
     signature.
@@ -351,7 +350,7 @@ int wc_ed25519_verify_msg(const byte* sig, word32 siglen, const byte* msg,
 
     \brief This function verifies the Ed25519 signature of a message to ensure
     authenticity. The context is included as part of the data
-    verified. It returns the answer through res, with 1 corresponding to
+    verified. It returns the answer through ret, with 1 corresponding to
     a valid signature, and 0 corresponding to an invalid signature.
 
     \return 0 Returned upon successfully performing the signature
@@ -365,7 +364,7 @@ int wc_ed25519_verify_msg(const byte* sig, word32 siglen, const byte* msg,
     \param [in] siglen Length of the signature to verify.
     \param [in] msg Pointer to the buffer containing the message to verify.
     \param [in] msgLen Length of the message to verify.
-    \param [out] res Pointer to the result of the verification. 1 indicates the
+    \param [out] ret Pointer to the result of the verification. 1 indicates the
     message was successfully verified.
     \param [in] key Pointer to a public Ed25519 key with which to verify the
     signature.
@@ -408,7 +407,7 @@ int wc_ed25519ctx_verify_msg(const byte* sig, word32 siglen, const byte* msg,
     message to ensure authenticity. The context is included as part of the data
     verified. The hash is the pre-hashed message before signature calculation.
     The hash algorithm used to create message digest must be SHA-512.
-    The answer is returned through res, with 1 corresponding to a valid
+    The answer is returned through ret, with 1 corresponding to a valid
     signature, and 0 corresponding to an invalid signature.
 
 
@@ -424,7 +423,7 @@ int wc_ed25519ctx_verify_msg(const byte* sig, word32 siglen, const byte* msg,
     \param [in] hash Pointer to the buffer containing the hash of the message
     to verify.
     \param [in] hashLen Length of the hash to verify.
-    \param [out] res Pointer to the result of the verification. 1 indicates the
+    \param [out] ret Pointer to the result of the verification. 1 indicates the
     message was successfully verified.
     \param [in] key Pointer to a public Ed25519 key with which to verify the
     signature.
@@ -441,8 +440,8 @@ int wc_ed25519ctx_verify_msg(const byte* sig, word32 siglen, const byte* msg,
     byte hash[] = { initialize with SHA-512 hash of message };
     byte context[] = { initialize with context of signature };
     // initialize key with received public key
-    ret = wc_ed25519ph_verify_hash(sig, sizeof(sig), msg, sizeof(msg),
-            &verified, &key, );
+    ret = wc_ed25519ph_verify_hash(sig, sizeof(sig), hash, sizeof(hash),
+            &verified, &key, context, sizeof(context));
     if (ret < 0) {
         // error performing verification
     } else if (verified == 0)
@@ -466,7 +465,7 @@ int wc_ed25519ph_verify_hash(const byte* sig, word32 siglen, const byte* hash,
     \brief This function verifies the Ed25519 signature of a message to ensure
     authenticity. The context is included as part of the data
     verified. The message is pre-hashed before verification. It returns the
-    answer through res, with 1 corresponding to a valid signature, and 0
+    answer through ret, with 1 corresponding to a valid signature, and 0
     corresponding to an invalid signature.
 
     \return 0 Returned upon successfully performing the signature
@@ -480,7 +479,7 @@ int wc_ed25519ph_verify_hash(const byte* sig, word32 siglen, const byte* hash,
     \param [in] siglen Length of the signature to verify.
     \param [in] msg Pointer to the buffer containing the message to verify.
     \param [in] msgLen Length of the message to verify.
-    \param [out] res Pointer to the result of the verification. 1 indicates the
+    \param [out] ret Pointer to the result of the verification. 1 indicates the
     message was successfully verified.
     \param [in] key Pointer to a public Ed25519 key with which to verify the
     signature.
@@ -497,8 +496,8 @@ int wc_ed25519ph_verify_hash(const byte* sig, word32 siglen, const byte* hash,
     byte msg[] = { initialize with message };
     byte context[] = { initialize with context of signature };
     // initialize key with received public key
-    ret = wc_ed25519ctx_verify_msg(sig, sizeof(sig), msg, sizeof(msg),
-            &verified, &key, );
+    ret = wc_ed25519ph_verify_msg(sig, sizeof(sig), msg, sizeof(msg),
+            &verified, &key, context, sizeof(context));
     if (ret < 0) {
         // error performing verification
     } else if (verified == 0)
@@ -507,8 +506,8 @@ int wc_ed25519ph_verify_hash(const byte* sig, word32 siglen, const byte* hash,
     \endcode
 
     \sa wc_ed25519_verify_msg
+    \sa wc_ed25519ctx_verify_msg
     \sa wc_ed25519ph_verify_hash
-    \sa wc_ed25519ph_verify_msg
     \sa wc_ed25519_sign_msg
 */
 
@@ -562,7 +561,7 @@ void wc_ed25519_free(ed25519_key* key);
 /*!
     \ingroup ED25519
 
-    \brief This function imports a public ed25519_key pair from a buffer
+    \brief This function imports a public ed25519_key from a buffer
     containing the public key. This function will handle both compressed and
     uncompressed keys. The public key is checked that it matches the private
     key when one is present.
@@ -600,7 +599,7 @@ int wc_ed25519_import_public(const byte* in, word32 inLen, ed25519_key* key);
 /*!
     \ingroup ED25519
 
-    \brief This function imports a public ed25519_key pair from a buffer
+    \brief This function imports a public ed25519_key from a buffer
     containing the public key. This function will handle both compressed and
     uncompressed keys. Check public key matches private key, when present,
     when not trusted.
@@ -644,13 +643,11 @@ int wc_ed25519_import_public_ex(const byte* in, word32 inLen, ed25519_key* key,
     buffer.
 
     \return 0 Returned on successfully importing the Ed25519 key.
-    \return BAD_FUNC_ARG Returned if in or key evaluate to NULL, or if
-    privSz is less than ED25519_KEY_SIZE.
+    \return BAD_FUNC_ARG Returned if priv or key evaluate to NULL, or if
+    privSz is not equal to ED25519_KEY_SIZE.
 
     \param [in] priv Pointer to the buffer containing the private key.
     \param [in] privSz Length of the private key.
-    \param [in] pub Pointer to the buffer containing the public key.
-    \param [in] pubSz Length of the public key.
     \param [in,out] key Pointer to the ed25519_key object in which to store the
     imported private key.
 
@@ -686,9 +683,9 @@ int wc_ed25519_import_private_only(const byte* priv, word32 privSz,
     checked against the private key.
 
     \return 0 Returned on successfully importing the ed25519_key.
-    \return BAD_FUNC_ARG Returned if in or key evaluate to NULL, or if
-    either privSz is less than ED25519_KEY_SIZE or pubSz is less than
-    ED25519_PUB_KEY_SIZE.
+    \return BAD_FUNC_ARG Returned if priv or key evaluate to NULL; or if
+    either privSz is not equal to ED25519_KEY_SIZE nor ED25519_PRV_KEY_SIZE, or
+    pubSz is less than ED25519_PUB_KEY_SIZE.
 
     \param [in] priv Pointer to the buffer containing the private key.
     \param [in] privSz Length of the private key.
@@ -730,9 +727,9 @@ int wc_ed25519_import_private_key(const byte* priv, word32 privSz,
     uncompressed keys. The public is checked against private key if not trusted.
 
     \return 0 Returned on successfully importing the ed25519_key.
-    \return BAD_FUNC_ARG Returned if in or key evaluate to NULL, or if
-    either privSz is less than ED25519_KEY_SIZE or pubSz is less than
-    ED25519_PUB_KEY_SIZE.
+    \return BAD_FUNC_ARG Returned if priv or key evaluate to NULL; or if
+    either privSz is not equal to ED25519_KEY_SIZE nor ED25519_PRV_KEY_SIZE, or
+    pubSz is less than ED25519_PUB_KEY_SIZE.
 
     \param [in] priv Pointer to the buffer containing the private key.
     \param [in] privSz Length of the private key.
@@ -750,7 +747,7 @@ int wc_ed25519_import_private_key(const byte* priv, word32 privSz,
 
     ed25519_key key;
     wc_ed25519_init_key(&key);
-    ret = wc_ed25519_import_private_key(priv, sizeof(priv), pub, sizeof(pub),
+    ret = wc_ed25519_import_private_key_ex(priv, sizeof(priv), pub, sizeof(pub),
             &key, 1);
     if (ret != 0) {
         // error importing key
@@ -770,14 +767,14 @@ int wc_ed25519_import_private_key_ex(const byte* priv, word32 privSz,
 /*!
     \ingroup ED25519
 
-    \brief This function exports the private key from an ed25519_key
+    \brief This function exports the public key from an ed25519_key
     structure. It stores the public key in the buffer out, and sets the bytes
     written to this buffer in outLen.
 
     \return 0 Returned upon successfully exporting the public key.
     \return BAD_FUNC_ARG Returned if any of the input values evaluate to NULL.
     \return BUFFER_E Returned if the buffer provided is not large enough to
-    store the private key. Upon returning this error, the function sets the
+    store the public key. Upon returning this error, the function sets the
     size required in outLen.
 
     \param [in] key Pointer to an ed25519_key structure from which to export the
@@ -804,10 +801,11 @@ int wc_ed25519_import_private_key_ex(const byte* priv, word32 privSz,
 
     \sa wc_ed25519_import_public
     \sa wc_ed25519_import_public_ex
+    \sa wc_ed25519_export_private
     \sa wc_ed25519_export_private_only
 */
 
-int wc_ed25519_export_public(ed25519_key* key, byte* out, word32* outLen);
+int wc_ed25519_export_public(const ed25519_key* key, byte* out, word32* outLen);
 
 /*!
     \ingroup ED25519
@@ -817,7 +815,7 @@ int wc_ed25519_export_public(ed25519_key* key, byte* out, word32* outLen);
     the bytes written to this buffer in outLen.
 
     \return 0 Returned upon successfully exporting the private key.
-    \return ECC_BAD_ARG_E Returned if any of the input values evaluate to NULL.
+    \return BAD_FUNC_ARG Returned if any of the input values evaluate to NULL.
     \return BUFFER_E Returned if the buffer provided is not large enough
     to store the private key.
 
@@ -843,11 +841,12 @@ int wc_ed25519_export_public(ed25519_key* key, byte* out, word32* outLen);
     \endcode
 
     \sa wc_ed25519_export_public
+    \sa wc_ed25519_export_private
     \sa wc_ed25519_import_private_key
     \sa wc_ed25519_import_private_key_ex
 */
 
-int wc_ed25519_export_private_only(ed25519_key* key, byte* out, word32* outLen);
+int wc_ed25519_export_private_only(const ed25519_key* key, byte* out, word32* outLen);
 
 /*!
     \ingroup ED25519
@@ -857,7 +856,7 @@ int wc_ed25519_export_private_only(ed25519_key* key, byte* out, word32* outLen);
     the bytes written to this buffer in outLen.
 
     \return 0 Returned upon successfully exporting the key pair.
-    \return ECC_BAD_ARG_E Returned if any of the input values evaluate to NULL.
+    \return BAD_FUNC_ARG Returned if any of the input values evaluate to NULL.
     \return BUFFER_E Returned if the buffer provided is not large enough
     to store the key pair.
 
@@ -891,7 +890,7 @@ int wc_ed25519_export_private_only(ed25519_key* key, byte* out, word32* outLen);
     \sa wc_ed25519_export_private_only
 */
 
-int wc_ed25519_export_private(ed25519_key* key, byte* out, word32* outLen);
+int wc_ed25519_export_private(const ed25519_key* key, byte* out, word32* outLen);
 
 /*!
     \ingroup ED25519
@@ -902,7 +901,7 @@ int wc_ed25519_export_private(ed25519_key* key, byte* out, word32* outLen);
     in the buffer pub, and sets the bytes written to this buffer in pubSz.
 
     \return 0 Returned upon successfully exporting the key pair.
-    \return ECC_BAD_ARG_E Returned if any of the input values evaluate to NULL.
+    \return BAD_FUNC_ARG Returned if any of the input values evaluate to NULL.
     \return BUFFER_E Returned if the buffer provided is not large enough
     to store the key pair.
 
@@ -938,7 +937,7 @@ int wc_ed25519_export_private(ed25519_key* key, byte* out, word32* outLen);
     \sa wc_ed25519_export_public
 */
 
-int wc_ed25519_export_key(ed25519_key* key,
+int wc_ed25519_export_key(const ed25519_key* key,
                           byte* priv, word32 *privSz,
                           byte* pub, word32 *pubSz);
 
@@ -949,7 +948,8 @@ int wc_ed25519_export_key(ed25519_key* key,
     the private key.
 
     \return 0 Returned if the private and public key matched.
-    \return BAD_FUNC_ARGS Returned if the given key is NULL.
+    \return BAD_FUNC_ARG Returned if the given key is NULL.
+    \return PUBLIC_KEY_E Returned if the no public key available or is invalid.
 
     \param [in] key Pointer to an ed25519_key structure holding a private and
     public key.
@@ -982,7 +982,7 @@ int wc_ed25519_check_key(ed25519_key* key);
     \brief This function returns the size of an Ed25519 - 32 bytes.
 
     \return ED25519_KEY_SIZE The size of a valid private key (32 bytes).
-    \return BAD_FUNC_ARGS Returned if the given key is NULL.
+    \return BAD_FUNC_ARG Returned if the given key is NULL.
 
     \param [in] key Pointer to an ed25519_key structure for which to get the
     key size.
@@ -1001,7 +1001,7 @@ int wc_ed25519_check_key(ed25519_key* key);
     \sa wc_ed25519_make_key
 */
 
-int wc_ed25519_size(ed25519_key* key);
+int wc_ed25519_size(const ed25519_key* key);
 
 /*!
     \ingroup ED25519
@@ -1030,7 +1030,7 @@ int wc_ed25519_size(ed25519_key* key);
     \sa wc_ed25519_pub_size
 */
 
-int wc_ed25519_priv_size(ed25519_key* key);
+int wc_ed25519_priv_size(const ed25519_key* key);
 
 /*!
     \ingroup ED25519
@@ -1058,7 +1058,7 @@ int wc_ed25519_priv_size(ed25519_key* key);
     \sa wc_ed25519_priv_size
 */
 
-int wc_ed25519_pub_size(ed25519_key* key);
+int wc_ed25519_pub_size(const ed25519_key* key);
 
 /*!
     \ingroup ED25519
@@ -1086,4 +1086,4 @@ int wc_ed25519_pub_size(ed25519_key* key);
     \sa wc_ed25519_sign_msg
 */
 
-int wc_ed25519_sig_size(ed25519_key* key);
+int wc_ed25519_sig_size(const ed25519_key* key);
