@@ -453,12 +453,12 @@ int wolfSSL_CertManagerUnloadCAs(WOLFSSL_CERT_MANAGER* cm)
     return ret;
 }
 
-static int wolfSSL_CertManagerUnloadIntermediateCertsEx(
+int wolfSSL_CertManagerUnloadTypeCerts(
                                 WOLFSSL_CERT_MANAGER* cm, byte type)
 {
     int ret = WOLFSSL_SUCCESS;
 
-    WOLFSSL_ENTER("wolfSSL_CertManagerUnloadIntermediateCertsEx");
+    WOLFSSL_ENTER("wolfSSL_CertManagerUnloadTypeCerts");
 
     /* Validate parameter. */
     if (cm == NULL) {
@@ -485,7 +485,7 @@ static int wolfSSL_CertManagerUnloadTempIntermediateCerts(
     WOLFSSL_CERT_MANAGER* cm)
 {
     WOLFSSL_ENTER("wolfSSL_CertManagerUnloadTempIntermediateCerts");
-    return wolfSSL_CertManagerUnloadIntermediateCertsEx(cm, WOLFSSL_TEMP_CA);
+    return wolfSSL_CertManagerUnloadTypeCerts(cm, WOLFSSL_TEMP_CA);
 }
 #endif
 
@@ -493,7 +493,7 @@ int wolfSSL_CertManagerUnloadIntermediateCerts(
     WOLFSSL_CERT_MANAGER* cm)
 {
     WOLFSSL_ENTER("wolfSSL_CertManagerUnloadIntermediateCerts");
-    return wolfSSL_CertManagerUnloadIntermediateCertsEx(cm, WOLFSSL_CHAIN_CA);
+    return wolfSSL_CertManagerUnloadTypeCerts(cm, WOLFSSL_CHAIN_CA);
 }
 
 #ifdef WOLFSSL_TRUST_PEER_CERT
@@ -530,7 +530,7 @@ int wolfSSL_CertManagerUnload_trust_peers(WOLFSSL_CERT_MANAGER* cm)
 }
 #endif /* WOLFSSL_TRUST_PEER_CERT */
 
-/* Load certificate/s from buffer with flags.
+/* Load certificate/s from buffer with flags and type.
  *
  * @param [in] cm         Certificate manager.
  * @param [in] buff       Buffer holding encoding of certificate.
@@ -544,17 +544,26 @@ int wolfSSL_CertManagerUnload_trust_peers(WOLFSSL_CERT_MANAGER* cm)
  *                          WOLFSSL_LOAD_FLAG_PEM_CA_ONLY,
  *                          WOLFSSL_LOAD_FLAG_IGNORE_BAD_PATH_ERR, and
  *                          WOLFSSL_LOAD_FLAG_IGNORE_ZEROFILE.
+ * @param [in] type       The CA cert's type, used in the internal CA
+                            table.  Defaults to WOLFSSL_USER_CA, passing
+                            in WOLFSSL_USER_CA = noop.  Recommended to
+                            set to WOLFSSL_USER_INTER when loading
+                            intermediate certs to allow unloading via
+                            wolfSSL_CertManagerUnloadTypeCerts.
  * @return  WOLFSSL_SUCCESS on success.
  * @return  WOLFSSL_FATAL_ERROR when cm is NULL or failed create WOLFSSL_CTX.
  * @return  Other values on loading failure.
  */
-int wolfSSL_CertManagerLoadCABuffer_ex(WOLFSSL_CERT_MANAGER* cm,
-    const unsigned char* buff, long sz, int format, int userChain, word32 flags)
+int wolfSSL_CertManagerLoadCABufferType(WOLFSSL_CERT_MANAGER* cm,
+    const unsigned char* buff, long sz, int format, int userChain,
+    word32 flags, int type)
 {
     int ret = WOLFSSL_SUCCESS;
     WOLFSSL_CTX* tmp = NULL;
+    DecodedCert* dCert = NULL;
+    DerBuffer* der = NULL;
 
-    WOLFSSL_ENTER("wolfSSL_CertManagerLoadCABuffer_ex");
+    WOLFSSL_ENTER("wolfSSL_CertManagerLoadCABufferType");
 
     /* Validate parameters. */
     if (cm == NULL) {
@@ -583,10 +592,81 @@ int wolfSSL_CertManagerLoadCABuffer_ex(WOLFSSL_CERT_MANAGER* cm,
         /* Clear certificate manager in WOLFSSL_CTX so it won't be freed. */
         tmp->cm = NULL;
     }
+    if (ret == WOLFSSL_SUCCESS && type != WOLFSSL_USER_CA) {
+        dCert = (DecodedCert*)XMALLOC(sizeof(DecodedCert), cm->heap,
+                                                DYNAMIC_TYPE_DCERT);
+
+        if (dCert == NULL) {
+            ret = WOLFSSL_FATAL_ERROR;
+        } else {
+            if (format == WOLFSSL_FILETYPE_PEM) {
+            #ifndef WOLFSSL_PEM_TO_DER
+                ret = NOT_COMPILED_IN;
+            #else
+                ret = PemToDer(buff, sz, CERT_TYPE, &der, cm->heap, NULL, NULL);
+                if (!ret) {
+                    /* Replace buffer pointer and size with DER buffer. */
+                    buff = der->buffer;
+                    sz = (long)der->length;
+                    ret = WOLFSSL_SUCCESS;
+                } else {
+                    WOLFSSL_ERROR(ret);
+                    ret = WOLFSSL_FATAL_ERROR;
+                }
+            #endif
+            }
+
+            if (ret == WOLFSSL_SUCCESS) {
+                XMEMSET(dCert, 0, sizeof(DecodedCert));
+                wc_InitDecodedCert(dCert, buff,
+                                (word32)sz, cm->heap);
+                ret = wc_ParseCert(dCert, CERT_TYPE, NO_VERIFY, NULL);
+                if (ret) {
+                    ret = WOLFSSL_FATAL_ERROR;
+                } else {
+                    ret = SetCAType(cm, dCert->extSubjKeyId, type);
+                }
+            }
+
+            if (dCert) {
+                wc_FreeDecodedCert(dCert);
+                XFREE(dCert, cm->heap, DYNAMIC_TYPE_DCERT);
+            }
+            if (der) {
+                FreeDer(&der);
+            }
+        }
+    }
 
     /* Dispose of temporary WOLFSSL_CTX. */
     wolfSSL_CTX_free(tmp);
     return ret;
+
+}
+
+/* Load certificate/s from buffer with flags.
+ *
+ * @param [in] cm         Certificate manager.
+ * @param [in] buff       Buffer holding encoding of certificate.
+ * @param [in] sz         Length in bytes of data in buffer.
+ * @param [in] format     Format of encoding. Valid values:
+ *                          WOLFSSL_FILETYPE_ASN1, WOLFSSL_FILETYPE_PEM.
+ * @param [in] userChain  Indicates buffer holds chain of certificates.
+ * @param [in] flags      Flags to modify behaviour of loading. Valid flags:
+ *                          WOLFSSL_LOAD_FLAG_IGNORE_ERR,
+ *                          WOLFSSL_LOAD_FLAG_DATE_ERR_OKAY,
+ *                          WOLFSSL_LOAD_FLAG_PEM_CA_ONLY,
+ *                          WOLFSSL_LOAD_FLAG_IGNORE_BAD_PATH_ERR, and
+ *                          WOLFSSL_LOAD_FLAG_IGNORE_ZEROFILE.
+ * @return  WOLFSSL_SUCCESS on success.
+ * @return  WOLFSSL_FATAL_ERROR when cm is NULL or failed create WOLFSSL_CTX.
+ * @return  Other values on loading failure.
+ */
+int wolfSSL_CertManagerLoadCABuffer_ex(WOLFSSL_CERT_MANAGER* cm,
+    const unsigned char* buff, long sz, int format, int userChain, word32 flags)
+{
+    return wolfSSL_CertManagerLoadCABufferType(cm, buff, sz, format, userChain,
+        flags, WOLFSSL_USER_CA);
 }
 
 /* Load certificate/s from buffer into table.
@@ -658,26 +738,19 @@ int CM_VerifyBuffer_ex(WOLFSSL_CERT_MANAGER* cm, const unsigned char* buff,
     int ret = 0;
     int fatal = 0;
     DerBuffer* der = NULL;
-#ifdef WOLFSSL_SMALL_STACK
-    DecodedCert* cert = NULL;
-#else
-    DecodedCert  cert[1];
-#endif
+    WC_DECLARE_VAR(cert, DecodedCert, 1, 0);
 
     WOLFSSL_ENTER("CM_VerifyBuffer_ex");
 
     (void)prev_err;
 
-#ifdef WOLFSSL_SMALL_STACK
     /* Allocate memory for decoded certificate. */
-    cert = (DecodedCert*)XMALLOC(sizeof(DecodedCert), cm->heap,
-         DYNAMIC_TYPE_DCERT);
-    if (cert == NULL) {
-        ret = MEMORY_E;
-        fatal = 1;
-    }
-    if (ret == 0)
-#endif
+    WC_ALLOC_VAR_EX(cert, DecodedCert, 1, cm->heap, DYNAMIC_TYPE_DCERT,
+    {
+        ret=MEMORY_E;
+        fatal=1;
+    });
+    if (WC_VAR_OK(cert))
     {
         /* Reset fields of decoded certificate. */
         XMEMSET(cert, 0, sizeof(DecodedCert));
@@ -726,11 +799,7 @@ int CM_VerifyBuffer_ex(WOLFSSL_CERT_MANAGER* cm, const unsigned char* buff,
 #ifndef NO_WOLFSSL_CM_VERIFY
     /* Use callback to perform verification too if available. */
     if ((!fatal) && cm->verifyCallback) {
-    #ifdef WOLFSSL_SMALL_STACK
-        ProcPeerCertArgs* args;
-    #else
-        ProcPeerCertArgs  args[1];
-    #endif
+        WC_DECLARE_VAR(args, ProcPeerCertArgs, 1, 0);
         buffer certBuf;
 
     #ifdef WOLFSSL_SMALL_STACK
@@ -763,19 +832,14 @@ int CM_VerifyBuffer_ex(WOLFSSL_CERT_MANAGER* cm, const unsigned char* buff,
             /* Use callback to verify certificate. */
             ret = DoVerifyCallback(cm, NULL, ret, args);
         }
-    #ifdef WOLFSSL_SMALL_STACK
-        /* Dispose of allocated callback args. */
-        XFREE(args, cm->heap, DYNAMIC_TYPE_TMP_BUFFER);
-    #endif
+        WC_FREE_VAR_EX(args, cm->heap, DYNAMIC_TYPE_TMP_BUFFER);
     }
 #endif
 
     /* Dispose of allocated memory. */
     FreeDecodedCert(cert);
     FreeDer(&der);
-#ifdef WOLFSSL_SMALL_STACK
-    XFREE(cert, cm->heap, DYNAMIC_TYPE_DCERT);
-#endif
+    WC_FREE_VAR_EX(cert, cm->heap, DYNAMIC_TYPE_DCERT);
 
     /* Convert the ret value to a return value. */
     return (ret == 0) ? WOLFSSL_SUCCESS : ret;
@@ -1800,11 +1864,7 @@ int wolfSSL_CertManagerCheckCRL(WOLFSSL_CERT_MANAGER* cm,
     const unsigned char* der, int sz)
 {
     int ret = 0;
-#ifdef WOLFSSL_SMALL_STACK
-    DecodedCert* cert = NULL;
-#else
-    DecodedCert  cert[1];
-#endif
+    WC_DECLARE_VAR(cert, DecodedCert, 1, 0);
 
     WOLFSSL_ENTER("wolfSSL_CertManagerCheckCRL");
 
@@ -1815,14 +1875,10 @@ int wolfSSL_CertManagerCheckCRL(WOLFSSL_CERT_MANAGER* cm,
 
     /* Check if CRL checking enabled. */
     if ((ret == 0) && cm->crlEnabled) {
-    #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for decoded certificate. */
-        cert = (DecodedCert*)XMALLOC(sizeof(DecodedCert), NULL,
-            DYNAMIC_TYPE_DCERT);
-        if (cert == NULL)
-            ret = MEMORY_E;
-        if (ret == 0)
-    #endif
+        WC_ALLOC_VAR_EX(cert, DecodedCert, 1, NULL, DYNAMIC_TYPE_DCERT,
+            ret=MEMORY_E);
+        if (WC_VAR_OK(cert))
         {
             /* Initialize decoded certificate with buffer. */
             InitDecodedCert(cert, der, (word32)sz, NULL);
@@ -1839,9 +1895,7 @@ int wolfSSL_CertManagerCheckCRL(WOLFSSL_CERT_MANAGER* cm,
 
             /* Dispose of dynamically allocated memory. */
             FreeDecodedCert(cert);
-        #ifdef WOLFSSL_SMALL_STACK
-            XFREE(cert, NULL, DYNAMIC_TYPE_DCERT);
-        #endif
+            WC_FREE_VAR_EX(cert, NULL, DYNAMIC_TYPE_DCERT);
         }
     }
 
@@ -2326,11 +2380,7 @@ int wolfSSL_CertManagerCheckOCSP(WOLFSSL_CERT_MANAGER* cm,
     const unsigned char* der, int sz)
 {
     int ret = 0;
-#ifdef WOLFSSL_SMALL_STACK
-    DecodedCert* cert = NULL;
-#else
-    DecodedCert  cert[1];
-#endif
+    WC_DECLARE_VAR(cert, DecodedCert, 1, 0);
 
     WOLFSSL_ENTER("wolfSSL_CertManagerCheckOCSP");
 
@@ -2341,15 +2391,10 @@ int wolfSSL_CertManagerCheckOCSP(WOLFSSL_CERT_MANAGER* cm,
 
     /* Check if OCSP checking enabled. */
     if ((ret == 0) && cm->ocspEnabled) {
-    #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for decoded certificate. */
-        cert = (DecodedCert*)XMALLOC(sizeof(DecodedCert), cm->heap,
-            DYNAMIC_TYPE_DCERT);
-        if (cert == NULL) {
-            ret = MEMORY_E;
-        }
-        if (ret == 0)
-    #endif
+        WC_ALLOC_VAR_EX(cert, DecodedCert, 1, cm->heap, DYNAMIC_TYPE_DCERT,
+            ret=MEMORY_E);
+        if (WC_VAR_OK(cert))
         {
             /* Initialize decoded certificate with buffer. */
             InitDecodedCert(cert, der, (word32)sz, NULL);
@@ -2366,9 +2411,7 @@ int wolfSSL_CertManagerCheckOCSP(WOLFSSL_CERT_MANAGER* cm,
 
             /* Dispose of dynamically allocated memory. */
             FreeDecodedCert(cert);
-        #ifdef WOLFSSL_SMALL_STACK
-            XFREE(cert, cm->heap, DYNAMIC_TYPE_DCERT);
-        #endif
+            WC_FREE_VAR_EX(cert, cm->heap, DYNAMIC_TYPE_DCERT);
         }
     }
 
