@@ -28,6 +28,10 @@
     #include <wolfcrypt/src/misc.c>
 #endif
 
+#ifdef WOLFSSL_ASYNC_CRYPT
+    #define WOLFSSL_SMALL_STACK
+#endif
+
 #include <wolfssl/wolfcrypt/rsa.h>
 #include <wolfssl/wolfcrypt/types.h>
 #include <tests/api/api.h>
@@ -106,8 +110,9 @@ int test_wc_RsaPrivateKeyDecode(void)
 int test_wc_RsaPublicKeyDecode(void)
 {
     EXPECT_DECLS;
-#if !defined(NO_RSA) && (defined(USE_CERT_BUFFERS_1024) || \
-        defined(USE_CERT_BUFFERS_2048)) && !defined(HAVE_FIPS)
+#if !defined(NO_RSA) && !defined(NO_SHA256) && \
+        (defined(USE_CERT_BUFFERS_1024) || defined(USE_CERT_BUFFERS_2048)) && \
+        !defined(HAVE_FIPS)
     RsaKey keyPub;
     byte*  tmp = NULL;
     word32 idx = 0;
@@ -616,53 +621,100 @@ int test_wc_RsaPSS_VerifyCheckInline(void)
 int test_wc_RsaKeyToDer(void)
 {
     EXPECT_DECLS;
-#if !defined(NO_RSA) && defined(WOLFSSL_KEY_GEN)
-    RsaKey genKey;
-    WC_RNG rng;
+#if !defined(NO_RSA) && \
+    (defined(WOLFSSL_KEY_GEN) || defined(WOLFSSL_KEY_TO_DER))
+    RsaKey key;
     byte*  der = NULL;
+    word32 derSz = 0;
+#if defined(WOLFSSL_KEY_GEN)
+    WC_RNG rng;
 #if (!defined(WOLFSSL_SP_MATH) || defined(WOLFSSL_SP_MATH_ALL)) && \
     (!defined(HAVE_FIPS_VERSION) || (HAVE_FIPS_VERSION < 4)) && \
     (defined(RSA_MIN_SIZE) && (RSA_MIN_SIZE <= 1024))
-    int     bits = 1024;
-    word32  derSz = 611;
-    /* (2 x 128) + 2 (possible leading 00) + (5 x 64) + 5 (possible leading 00)
-       + 3 (e) + 8 (ASN tag) + 10 (ASN length) + 4 seqSz + 3 version */
+    int    bits = 1024;
 #else
-    int     bits = 2048;
-    word32  derSz = 1196;
-    /* (2 x 256) + 2 (possible leading 00) + (5 x 128) + 5 (possible leading 00)
-       + 3 (e) + 8 (ASN tag) + 17 (ASN length) + 4 seqSz + 3 version */
+    int    bits = 2048;
+#endif
+#else
+    word32 idx = 0;
+    byte* key_der = NULL;
+#if !defined(NO_FILESYSTEM)
+    const char* key_fname = "./certs/client-key.der";
+    XFILE file = XBADFILE;
+#endif
+#endif /* WOLFSSL_KEY_GEN */
+
+#if defined(WOLFSSL_KEY_GEN)
+    XMEMSET(&rng, 0, sizeof(rng));
+#endif
+    XMEMSET(&key, 0, sizeof(key));
+
+    /* Init RSA structure */
+    ExpectIntEQ(wc_InitRsaKey(&key, HEAP_HINT), 0);
+
+#if defined(WOLFSSL_KEY_GEN)
+    /* Init RMG */
+    ExpectIntEQ(wc_InitRng(&rng), 0);
+    /* Make key */
+    ExpectIntEQ(MAKE_RSA_KEY(&key, bits, WC_RSA_EXPONENT, &rng), 0);
+#else
+    /* Import a key */
+#if !defined(NO_FILESYSTEM)
+    ExpectTrue((file = XFOPEN(key_fname, "rb")) != XBADFILE);
+    ExpectIntEQ(XFSEEK(file, 0, XSEEK_END), 0);
+    ExpectIntGT(derSz = (word32)XFTELL(file), 0);
+    ExpectIntEQ(XFSEEK(file, 0, XSEEK_SET), 0);
+    ExpectNotNull(key_der = (byte*)XMALLOC(derSz, NULL,
+        DYNAMIC_TYPE_TMP_BUFFER));
+    ExpectIntEQ((int)XFREAD(key_der, 1, derSz, file), derSz);
+    XFCLOSE(file);
+#elif defined(USE_CERT_BUFFERS_1024) && \
+    (defined(RSA_MIN_SIZE) && (RSA_MIN_SIZE <= 1024))
+    key_der = (byte*)client_key_der_1024;
+    derSz = (word32)sizeof_client_key_der_1024;
+#elif defined(USE_CERT_BUFFERS_2048)
+    key_der = (byte*)client_key_der_2048;
+    derSz = (word32)sizeof_client_key_der_2048;
+#elif defined(USE_CERT_BUFFERS_3072)
+    key_der = (byte*)client_key_der_3072;
+    derSz = (word32)sizeof_client_key_der_3072;
+#elif defined(USE_CERT_BUFFERS_4096)
+    key_der = (byte*)client_key_der_4096;
+    derSz = (word32)sizeof_client_key_der_4096;
 #endif
 
-    XMEMSET(&rng, 0, sizeof(rng));
-    XMEMSET(&genKey, 0, sizeof(genKey));
+    /* Import private key */
+    ExpectIntEQ(wc_RsaPrivateKeyDecode(key_der, &idx, &key, derSz), 0);
 
+#if !defined(NO_FILESYSTEM)
+    XFREE(key_der, NULL, DYNAMIC_TYPE_TMP_BUFFER);
+#endif
+#endif /* WOLFSSL_KEY_GEN */
+
+    /* Get output length */
+    ExpectIntGT((derSz = wc_RsaKeyToDer(&key, NULL, 0)), 0);
     ExpectNotNull(der = (byte*)XMALLOC(derSz, NULL, DYNAMIC_TYPE_TMP_BUFFER));
-    /* Init structures. */
-    ExpectIntEQ(wc_InitRsaKey(&genKey, HEAP_HINT), 0);
-    ExpectIntEQ(wc_InitRng(&rng), 0);
-    /* Make key. */
-    ExpectIntEQ(MAKE_RSA_KEY(&genKey, bits, WC_RSA_EXPONENT, &rng), 0);
 
-    ExpectIntGT(wc_RsaKeyToDer(&genKey, der, derSz), 0);
+    /* Test exporting private key to DER */
+    ExpectIntGT(wc_RsaKeyToDer(&key, der, derSz), 0);
 
     /* Pass good/bad args. */
-    ExpectIntEQ(wc_RsaKeyToDer(NULL, der, FOURK_BUF),
+    ExpectIntEQ(wc_RsaKeyToDer(NULL, der, derSz),
         WC_NO_ERR_TRACE(BAD_FUNC_ARG));
-    /* Get just the output length */
-    ExpectIntGT(wc_RsaKeyToDer(&genKey, NULL, 0), 0);
     /* Try Public Key. */
-    genKey.type = 0;
-    ExpectIntEQ(wc_RsaKeyToDer(&genKey, der, FOURK_BUF),
+    key.type = 0;
+    ExpectIntEQ(wc_RsaKeyToDer(&key, der, derSz),
         WC_NO_ERR_TRACE(BAD_FUNC_ARG));
     #ifdef WOLFSSL_CHECK_MEM_ZERO
         /* Put back to Private Key */
-        genKey.type = 1;
+        key.type = 1;
     #endif
 
     XFREE(der, NULL, DYNAMIC_TYPE_TMP_BUFFER);
-    DoExpectIntEQ(wc_FreeRsaKey(&genKey), 0);
-    DoExpectIntEQ(wc_FreeRng(&rng), 0);
+    DoExpectIntEQ(wc_FreeRsaKey(&key), 0);
+    #if defined(WOLFSSL_KEY_GEN)
+        DoExpectIntEQ(wc_FreeRng(&rng), 0);
+    #endif
 #endif
     return EXPECT_RESULT();
 } /* END test_wc_RsaKeyToDer */
@@ -724,7 +776,8 @@ int test_wc_RsaKeyToPublicDer(void)
 int test_wc_RsaPublicEncryptDecrypt(void)
 {
     EXPECT_DECLS;
-#if !defined(NO_RSA) && defined(WOLFSSL_KEY_GEN)
+#if !defined(NO_RSA) && defined(WOLFSSL_KEY_GEN) && \
+    !defined(WOLFSSL_RSA_PUBLIC_ONLY)
     RsaKey key;
     WC_RNG rng;
     const char inStr[] = TEST_STRING;
@@ -793,14 +846,16 @@ int test_wc_RsaPublicEncryptDecrypt_ex(void)
     WC_RNG  rng;
     const char inStr[] = TEST_STRING;
     const word32 inLen = (word32)TEST_STRING_SZ;
-    const word32 plainSz = (word32)TEST_STRING_SZ;
-    byte*   res = NULL;
     int     idx = 0;
     int          bits = TEST_RSA_BITS;
     const word32 cipherSz = TEST_RSA_BYTES;
+#ifndef WOLFSSL_RSA_PUBLIC_ONLY
+    const word32 plainSz = (word32)TEST_STRING_SZ;
+    byte*   res = NULL;
 
-    WC_DECLARE_VAR(in, byte, TEST_STRING_SZ, NULL);
     WC_DECLARE_VAR(plain, byte, TEST_STRING_SZ, NULL);
+#endif
+    WC_DECLARE_VAR(in, byte, TEST_STRING_SZ, NULL);
     WC_DECLARE_VAR(cipher, byte, TEST_RSA_BYTES, NULL);
 
     WC_ALLOC_VAR(in, byte, TEST_STRING_SZ, NULL);
