@@ -1,12 +1,12 @@
 /* coding.c
  *
- * Copyright (C) 2006-2023 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -19,18 +19,11 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
-
-#ifdef HAVE_CONFIG_H
-    #include <config.h>
-#endif
-
-#include <wolfssl/wolfcrypt/settings.h>
+#include <wolfssl/wolfcrypt/libwolfssl_sources.h>
 
 #ifndef NO_CODING
 
 #include <wolfssl/wolfcrypt/coding.h>
-#include <wolfssl/wolfcrypt/error-crypt.h>
-#include <wolfssl/wolfcrypt/logging.h>
 #ifndef NO_ASN
     #include <wolfssl/wolfcrypt/asn.h> /* For PEM_LINE_SZ */
 #endif
@@ -59,23 +52,33 @@ enum {
 
 #ifdef WOLFSSL_BASE64_DECODE
 
-#ifdef BASE64_NO_TABLE
-static WC_INLINE byte Base64_Char2Val(byte c)
+static WC_INLINE byte Base64_Char2Val_CT(byte c)
 {
-    word16 v = 0x0000;
+    int v;
+    int smallEnd   = (int)c - 0x7b;
+    int smallStart = (int)c - 0x61;
+    int bigEnd     = (int)c - 0x5b;
+    int bigStart   = (int)c - 0x41;
+    int numEnd     = (int)c - 0x3a;
+    int numStart   = (int)c - 0x30;
+    int slashEnd   = (int)c - 0x30;
+    int slashStart = (int)c - 0x2f;
+    int plusEnd    = (int)c - 0x2c;
+    int plusStart  = (int)c - 0x2b;
 
-    v |= 0xff3E & ctMask16Eq(c, 0x2b);
-    v |= 0xff3F & ctMask16Eq(c, 0x2f);
-    v |= (c + 0xff04) & ctMask16GTE(c, 0x30) & ctMask16LTE(c, 0x39);
-    v |= (0xff00 + c - 0x41) & ctMask16GTE(c, 0x41) & ctMask16LTE(c, 0x5a);
-    v |= (0xff00 + c - 0x47) & ctMask16GTE(c, 0x61) & ctMask16LTE(c, 0x7a);
-    v |= ~(v >> 8);
+    v  = ((smallStart >> 8) ^ (smallEnd >> 8)) & (smallStart + 26 + 1);
+    v |= ((bigStart   >> 8) ^ (bigEnd   >> 8)) & (bigStart   +  0 + 1);
+    v |= ((numStart   >> 8) ^ (numEnd   >> 8)) & (numStart   + 52 + 1);
+    v |= ((slashStart >> 8) ^ (slashEnd >> 8)) & (slashStart + 63 + 1);
+    v |= ((plusStart  >> 8) ^ (plusEnd  >> 8)) & (plusStart  + 62 + 1);
 
-    return (byte)v;
+    return (byte)(v - 1);
 }
-#else
+
+#ifndef BASE64_NO_TABLE
+
 static
-ALIGN64 const byte base64Decode[] = {          /* + starts at 0x2B */
+ALIGN64 const byte base64Decode_table[] = {    /* + starts at 0x2B */
 /* 0x28:       + , - . / */                   62, BAD, BAD, BAD,  63,
 /* 0x30: 0 1 2 3 4 5 6 7 */    52,  53,  54,  55,  56,  57,  58,  59,
 /* 0x38: 8 9 : ; < = > ? */    60,  61, BAD, BAD, BAD, BAD, BAD, BAD,
@@ -88,30 +91,31 @@ ALIGN64 const byte base64Decode[] = {          /* + starts at 0x2B */
 /* 0x70: p q r s t u v w */    41,  42,  43,  44,  45,  46,  47,  48,
 /* 0x78: x y z           */    49,  50,  51
                             };
-#define BASE64DECODE_SZ    (byte)(sizeof(base64Decode))
+#define BASE64DECODE_TABLE_SZ    (byte)(sizeof(base64Decode_table))
 
-static WC_INLINE byte Base64_Char2Val(byte c)
+static WC_INLINE byte Base64_Char2Val_by_table(byte c)
 {
-#ifndef WC_NO_CACHE_RESISTANT
+#ifdef WC_CACHE_RESISTANT_BASE64_TABLE
     /* 80 characters in table.
      * 64 bytes in a cache line - first line has 64, second has 16
      */
     byte v;
     byte mask;
 
-    c -= BASE64_MIN;
+    c = (byte)(c - BASE64_MIN);
     mask = (byte)((((byte)(0x3f - c)) >> 7) - 1);
     /* Load a value from the first cache line and use when mask set. */
-    v  = (byte)(base64Decode[ c & 0x3f        ] &   mask);
+    v  = (byte)(base64Decode_table[ c & 0x3f        ] &   mask);
     /* Load a value from the second cache line and use when mask not set. */
-    v |= (byte)(base64Decode[(c & 0x0f) | 0x40] & (~mask));
+    v |= (byte)(base64Decode_table[(c & 0x0f) | 0x40] & (~mask));
 
     return v;
 #else
-    return base64Decode[c - BASE64_MIN];
+    return base64Decode_table[c - BASE64_MIN];
 #endif
 }
-#endif
+
+#endif /* !BASE64_NO_TABLE */
 
 int Base64_SkipNewline(const byte* in, word32 *inLen,
   word32 *outJ)
@@ -161,18 +165,14 @@ int Base64_SkipNewline(const byte* in, word32 *inLen,
     return 0;
 }
 
-int Base64_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
+#ifndef BASE64_NO_TABLE
+
+int Base64_Decode_nonCT(const byte* in, word32 inLen, byte* out, word32* outLen)
 {
     word32 i = 0;
     word32 j = 0;
-    word32 plainSz = inLen - ((inLen + (BASE64_LINE_SZ - 1)) / BASE64_LINE_SZ );
     int ret;
-#ifndef BASE64_NO_TABLE
-    const byte maxIdx = BASE64DECODE_SZ + BASE64_MIN - 1;
-#endif
-
-    plainSz = (plainSz * 3 + 3) / 4;
-    if (plainSz > *outLen) return BAD_FUNC_ARG;
+    const byte maxIdx = BASE64DECODE_TABLE_SZ + BASE64_MIN - 1;
 
     while (inLen > 3) {
         int pad3 = 0;
@@ -216,7 +216,6 @@ int Base64_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
         if (pad3 && !pad4)
             return ASN_INPUT_E;
 
-#ifndef BASE64_NO_TABLE
         if (e1 < BASE64_MIN || e2 < BASE64_MIN || e3 < BASE64_MIN ||
                                                               e4 < BASE64_MIN) {
             WOLFSSL_MSG("Bad Base64 Decode data, too small");
@@ -227,17 +226,16 @@ int Base64_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
             WOLFSSL_MSG("Bad Base64 Decode data, too big");
             return ASN_INPUT_E;
         }
-#endif
 
         if (i + 1 + !pad3 + !pad4 > *outLen) {
             WOLFSSL_MSG("Bad Base64 Decode out buffer, too small");
-            return BAD_FUNC_ARG;
+            return BUFFER_E;
         }
 
-        e1 = Base64_Char2Val(e1);
-        e2 = Base64_Char2Val(e2);
-        e3 = (byte)((e3 == PAD) ? 0 : Base64_Char2Val(e3));
-        e4 = (byte)((e4 == PAD) ? 0 : Base64_Char2Val(e4));
+        e1 = Base64_Char2Val_by_table(e1);
+        e2 = Base64_Char2Val_by_table(e2);
+        e3 = (byte)((e3 == PAD) ? 0 : Base64_Char2Val_by_table(e3));
+        e4 = (byte)((e4 == PAD) ? 0 : Base64_Char2Val_by_table(e4));
 
         if (e1 == BAD || e2 == BAD || e3 == BAD || e4 == BAD) {
             WOLFSSL_MSG("Bad Base64 Decode bad character");
@@ -256,15 +254,111 @@ int Base64_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
         else
             break;
     }
-/* If the output buffer has a room for an extra byte, add a null terminator */
+
+    /* If the output buffer has a room for an extra byte, add a null terminator */
     if (out && *outLen > i)
         out[i]= '\0';
 
+    /* Note, *outLen won't reflect the optional terminating null. */
     *outLen = i;
 
     return 0;
 }
 
+#endif /* !BASE64_NO_TABLE */
+
+int Base64_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
+{
+    word32 i = 0;
+    word32 j = 0;
+    int ret;
+
+    while (inLen > 3) {
+        int pad3 = 0;
+        int pad4 = 0;
+        byte b1, b2, b3;
+        byte e1, e2, e3, e4;
+
+        if ((ret = Base64_SkipNewline(in, &inLen, &j)) != 0) {
+            if (ret == WC_NO_ERR_TRACE(BUFFER_E)) {
+                /* Running out of buffer here is not an error */
+                break;
+            }
+            return ret;
+        }
+        e1 = in[j++];
+        if (e1 == '\0') {
+            break;
+        }
+        inLen--;
+        if ((ret = Base64_SkipNewline(in, &inLen, &j)) != 0) {
+            return ret;
+        }
+        e2 = in[j++];
+        inLen--;
+        if ((ret = Base64_SkipNewline(in, &inLen, &j)) != 0) {
+            return ret;
+        }
+        e3 = in[j++];
+        inLen--;
+        if ((ret = Base64_SkipNewline(in, &inLen, &j)) != 0) {
+            return ret;
+        }
+        e4 = in[j++];
+        inLen--;
+
+        if (e3 == PAD)
+            pad3 = 1;
+        if (e4 == PAD)
+            pad4 = 1;
+
+        if (pad3 && !pad4)
+            return ASN_INPUT_E;
+
+        if (i + 1 + !pad3 + !pad4 > *outLen) {
+            WOLFSSL_MSG("Bad Base64 Decode out buffer, too small");
+            return BUFFER_E;
+        }
+
+        e1 = Base64_Char2Val_CT(e1);
+        e2 = Base64_Char2Val_CT(e2);
+        e3 = (byte)((e3 == PAD) ? 0 : Base64_Char2Val_CT(e3));
+        e4 = (byte)((e4 == PAD) ? 0 : Base64_Char2Val_CT(e4));
+
+        if (e1 == BAD || e2 == BAD || e3 == BAD || e4 == BAD) {
+            WOLFSSL_MSG("Bad Base64 Decode bad character");
+            return ASN_INPUT_E;
+        }
+
+        b1 = (byte)((e1 << 2) | (e2 >> 4));
+        b2 = (byte)(((e2 & 0xF) << 4) | (e3 >> 2));
+        b3 = (byte)(((e3 & 0x3) << 6) | e4);
+
+        out[i++] = b1;
+        if (!pad3)
+            out[i++] = b2;
+        if (!pad4)
+            out[i++] = b3;
+        else
+            break;
+    }
+
+    /* If the output buffer has a room for an extra byte, add a null terminator */
+    if (out && *outLen > i)
+        out[i]= '\0';
+
+    /* Note, *outLen won't reflect the optional terminating null. */
+    *outLen = i;
+
+    return 0;
+}
+
+#ifdef BASE64_NO_TABLE
+int Base64_Decode_nonCT(const byte* in, word32 inLen, byte* out, word32* outLen) {
+    return Base64_Decode(in, inLen, out, outLen);
+}
+#endif /* BASE64_NO_TABLE */
+
 #endif /* WOLFSSL_BASE64_DECODE */
 
 #if defined(WOLFSSL_BASE64_ENCODE)
@@ -297,8 +391,10 @@ static int CEscape(int escaped, byte e, byte* out, word32* i, word32 maxSz,
 
     if (raw)
         basic = e;
-    else
+    else if (e < sizeof(base64Encode))
         basic = base64Encode[e];
+    else
+        return BAD_FUNC_ARG;
 
     /* check whether to escape. Only escape for EncodeEsc */
     if (escaped == WC_ESC_NL_ENC) {
@@ -458,7 +554,7 @@ static int DoBase64_Encode(const byte* in, word32 inLen, byte* out,
     *outLen = i;
 
     if (ret == 0)
-        return getSzOnly ? LENGTH_ONLY_E : 0;
+        return getSzOnly ? WC_NO_ERR_TRACE(LENGTH_ONLY_E) : 0;
 
     return ret;
 }
@@ -488,7 +584,7 @@ int Base64_Encode_NoNl(const byte* in, word32 inLen, byte* out, word32* outLen)
 #ifdef WOLFSSL_BASE16
 
 static
-const byte hexDecode[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
+const ALIGN64 byte hexDecode[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
                            BAD, BAD, BAD, BAD, BAD, BAD, BAD,
                            10, 11, 12, 13, 14, 15,  /* upper case A-F */
                            BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,
@@ -507,7 +603,7 @@ int Base16_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
         return BAD_FUNC_ARG;
 
     if (inLen == 1 && *outLen && in) {
-        byte b = in[inIdx++] - BASE16_MIN;  /* 0 starts at 0x30 */
+        byte b = (byte)(in[inIdx++] - BASE16_MIN);  /* 0 starts at 0x30 */
 
         /* sanity check */
         if (b >=  sizeof(hexDecode)/sizeof(hexDecode[0]))
@@ -528,11 +624,11 @@ int Base16_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
         return BAD_FUNC_ARG;
 
     if (*outLen < (inLen / 2))
-        return BAD_FUNC_ARG;
+        return BUFFER_E;
 
     while (inLen) {
-        byte b  = in[inIdx++] - BASE16_MIN;  /* 0 starts at 0x30 */
-        byte b2 = in[inIdx++] - BASE16_MIN;
+        byte b  = (byte)(in[inIdx++] - BASE16_MIN);  /* 0 starts at 0x30 */
+        byte b2 = (byte)(in[inIdx++] - BASE16_MIN);
 
         /* sanity checks */
         if (b >=  sizeof(hexDecode)/sizeof(hexDecode[0]))
@@ -554,6 +650,11 @@ int Base16_Decode(const byte* in, word32 inLen, byte* out, word32* outLen)
     return 0;
 }
 
+static
+const ALIGN64 byte hexEncode[] = { '0', '1', '2', '3', '4', '5', '6', '7',
+                                   '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
+};
+
 int Base16_Encode(const byte* in, word32 inLen, byte* out, word32* outLen)
 {
     word32 outIdx = 0;
@@ -569,15 +670,8 @@ int Base16_Encode(const byte* in, word32 inLen, byte* out, word32* outLen)
         byte hb = in[i] >> 4;
         byte lb = in[i] & 0x0f;
 
-        /* ASCII value */
-        hb += '0';
-        if (hb > '9')
-            hb += 7;
-
-        /* ASCII value */
-        lb += '0';
-        if (lb>'9')
-            lb += 7;
+        hb = hexEncode[hb];
+        lb = hexEncode[lb];
 
         out[outIdx++] = hb;
         out[outIdx++] = lb;
