@@ -36,42 +36,47 @@ void fe_init(void)
  */
 void fe_frombytes(fe out, const unsigned char *in)
 {
-    out[0] = (((sword64)((in[ 0]      )       ))      )
-           | (((sword64)((in[ 1]      )       )) <<  8)
-           | (((sword64)((in[ 2]      )       )) << 16)
-           | (((sword64)((in[ 3]      )       )) << 24)
-           | (((sword64)((in[ 4]      )       )) << 32)
-           | (((sword64)((in[ 5]      )       )) << 40)
-           | (((sword64)((in[ 6]      ) & 0x07)) << 48);
-    out[1] = (((sword64)((in[ 6] >>  3) & 0x1f))      )
-           | (((sword64)((in[ 7]      )       )) <<  5)
-           | (((sword64)((in[ 8]      )       )) << 13)
-           | (((sword64)((in[ 9]      )       )) << 21)
-           | (((sword64)((in[10]      )       )) << 29)
-           | (((sword64)((in[11]      )       )) << 37)
-           | (((sword64)((in[12]      ) & 0x3f)) << 45);
-    out[2] = (((sword64)((in[12] >>  6) & 0x03))      )
-           | (((sword64)((in[13]      )       )) <<  2)
-           | (((sword64)((in[14]      )       )) << 10)
-           | (((sword64)((in[15]      )       )) << 18)
-           | (((sword64)((in[16]      )       )) << 26)
-           | (((sword64)((in[17]      )       )) << 34)
-           | (((sword64)((in[18]      )       )) << 42)
-           | (((sword64)((in[19]      ) & 0x01)) << 50);
-    out[3] = (((sword64)((in[19] >>  1) & 0x7f))      )
-           | (((sword64)((in[20]      )       )) <<  7)
-           | (((sword64)((in[21]      )       )) << 15)
-           | (((sword64)((in[22]      )       )) << 23)
-           | (((sword64)((in[23]      )       )) << 31)
-           | (((sword64)((in[24]      )       )) << 39)
-           | (((sword64)((in[25]      ) & 0x0f)) << 47);
-    out[4] = (((sword64)((in[25] >>  4) & 0x0f))      )
-           | (((sword64)((in[26]      )       )) <<  4)
-           | (((sword64)((in[27]      )       )) << 12)
-           | (((sword64)((in[28]      )       )) << 20)
-           | (((sword64)((in[29]      )       )) << 28)
-           | (((sword64)((in[30]      )       )) << 36)
-           | (((sword64)((in[31]      ) & 0x7f)) << 44);
+    out[0] = (sword64)(
+             (((word64)((in[ 0]      )       ))      )
+           | (((word64)((in[ 1]      )       )) <<  8)
+           | (((word64)((in[ 2]      )       )) << 16)
+           | (((word64)((in[ 3]      )       )) << 24)
+           | (((word64)((in[ 4]      )       )) << 32)
+           | (((word64)((in[ 5]      )       )) << 40)
+           | (((word64)((in[ 6]      ) & 0x07)) << 48));
+    out[1] = (sword64)(
+             (((word64)((in[ 6] >>  3) & 0x1f))      )
+           | (((word64)((in[ 7]      )       )) <<  5)
+           | (((word64)((in[ 8]      )       )) << 13)
+           | (((word64)((in[ 9]      )       )) << 21)
+           | (((word64)((in[10]      )       )) << 29)
+           | (((word64)((in[11]      )       )) << 37)
+           | (((word64)((in[12]      ) & 0x3f)) << 45));
+    out[2] = (sword64)(
+             (((word64)((in[12] >>  6) & 0x03))      )
+           | (((word64)((in[13]      )       )) <<  2)
+           | (((word64)((in[14]      )       )) << 10)
+           | (((word64)((in[15]      )       )) << 18)
+           | (((word64)((in[16]      )       )) << 26)
+           | (((word64)((in[17]      )       )) << 34)
+           | (((word64)((in[18]      )       )) << 42)
+           | (((word64)((in[19]      ) & 0x01)) << 50));
+    out[3] = (sword64)(
+             (((word64)((in[19] >>  1) & 0x7f))      )
+           | (((word64)((in[20]      )       )) <<  7)
+           | (((word64)((in[21]      )       )) << 15)
+           | (((word64)((in[22]      )       )) << 23)
+           | (((word64)((in[23]      )       )) << 31)
+           | (((word64)((in[24]      )       )) << 39)
+           | (((word64)((in[25]      ) & 0x0f)) << 47));
+    out[4] = (sword64)(
+             (((word64)((in[25] >>  4) & 0x0f))      )
+           | (((word64)((in[26]      )       )) <<  4)
+           | (((word64)((in[27]      )       )) << 12)
+           | (((word64)((in[28]      )       )) << 20)
+           | (((word64)((in[29]      )       )) << 28)
+           | (((word64)((in[30]      )       )) << 36)
+           | (((word64)((in[31]      ) & 0x7f)) << 44));
 }
 
 /* Convert a number represented as an array of words to an array of bytes.
