@@ -43,7 +43,7 @@
 
 #include <wolfssl/wolfcrypt/memory.h>
 #include <wolfssl/wolfcrypt/error-crypt.h>
-#include <wolfssl/wolfcrypt/port/Renesas/renesas-tsip-crypt.h>
+#include <wolfssl/wolfcrypt/port/Renesas/renesas_tsip_internal.h>
 
 extern struct WOLFSSL_HEAP_HINT* tsip_heap_hint;
 
@@ -82,7 +82,7 @@ WOLFSSL_LOCAL int tsip_Tls13GetHmacMessages(struct WOLFSSL* ssl, byte* mac)
         if (tuc == NULL) {
             ret = CRYPTOCB_UNAVAILABLE;
         }
-        else if (!tuc->HandshakeClientTrafficKey_set) {
+        else if (!tuc->internal->HandshakeClientTrafficKey_set) {
             WOLFSSL_MSG("Client handshake traffic keys aren't created by TSIP");
             ret = CRYPTOCB_UNAVAILABLE;
         }
@@ -97,8 +97,9 @@ WOLFSSL_LOCAL int tsip_Tls13GetHmacMessages(struct WOLFSSL* ssl, byte* mac)
     if (ret == 0) {
         if ((ret = tsip_hw_lock()) == 0) {
 
-            err = R_TSIP_Sha256HmacGenerateInit(&(tuc->hmacFinished13Handle),
-                                                &(tuc->clientFinished13Idx));
+            err = R_TSIP_Sha256HmacGenerateInit(
+                    &(tuc->internal->hmacFinished13Handle),
+                    &(tuc->internal->clientFinished13Idx));
 
             if (err != TSIP_SUCCESS) {
                 WOLFSSL_MSG("R_TSIP_Sha256HmacGenerateInit failed");
@@ -108,9 +109,9 @@ WOLFSSL_LOCAL int tsip_Tls13GetHmacMessages(struct WOLFSSL* ssl, byte* mac)
             if (ret == 0) {
 
                 err = R_TSIP_Sha256HmacGenerateUpdate(
-                                                &(tuc->hmacFinished13Handle),
-                                                (uint8_t*)hash,
-                                                WC_SHA256_DIGEST_SIZE);
+                                        &(tuc->internal->hmacFinished13Handle),
+                                        (uint8_t*)hash,
+                                        WC_SHA256_DIGEST_SIZE);
 
                 if (err != TSIP_SUCCESS) {
                     WOLFSSL_MSG("R_TSIP_Sha256HmacGenerateUpdate failed");
@@ -120,7 +121,7 @@ WOLFSSL_LOCAL int tsip_Tls13GetHmacMessages(struct WOLFSSL* ssl, byte* mac)
 
             if (ret == 0) {
                 err = R_TSIP_Sha256HmacGenerateFinal(
-                                            &(tuc->hmacFinished13Handle), mac);
+                                &(tuc->internal->hmacFinished13Handle), mac);
                 if (err != TSIP_SUCCESS) {
                     WOLFSSL_MSG("R_TSIP_Sha256HmacGenerateFinal failed");
                     ret = WC_HW_E;
@@ -185,7 +186,7 @@ WOLFSSL_LOCAL int tsip_StoreMessage(struct WOLFSSL* ssl, const byte* data,
 
     /* check if TSIP is used for this session */
     if (ret == 0) {
-        if (!tuc->Dhe_key_set) {
+        if (!tuc->internal->Dhe_key_set) {
             WOLFSSL_MSG("DH key not set.");
             ret = CRYPTOCB_UNAVAILABLE;
         }
@@ -195,7 +196,7 @@ WOLFSSL_LOCAL int tsip_StoreMessage(struct WOLFSSL* ssl, const byte* data,
     if (ret == 0) {
         c24to32(&data[1], &messageSz);
 
-        bag = &(tuc->messageBag);
+        bag = &(tuc->internal->messageBag);
 
         if (bag->msgIdx +1 > MAX_MSGBAG_MESSAGES ||
             bag->buffIdx + sz > MSGBAG_SIZE) {
@@ -246,7 +247,7 @@ WOLFSSL_LOCAL int tsip_GetMessageSha256(struct WOLFSSL* ssl, byte* hash,
         if (tuc == NULL) {
             ret = CRYPTOCB_UNAVAILABLE;
         }
-        bag = &(tuc->messageBag);
+        bag = &(tuc->internal->messageBag);
     }
 
     if (ret == 0) {
