@@ -27,6 +27,10 @@
     #undef WOLFSSL_RISCV_ASM
 #endif
 
+#if defined(WOLFSSL_PSOC6_CRYPTO)
+    #include <wolfssl/wolfcrypt/port/cypress/psoc6_crypto.h>
+#endif
+
 #if defined(WOLFSSL_SHA3) && !defined(WOLFSSL_XILINX_CRYPT) && \
    !defined(WOLFSSL_AFALG_XILINX_SHA3)
 
@@ -67,8 +71,7 @@
         defined(WOLFSSL_ARMASM))
     #include <wolfssl/wolfcrypt/cpuid.h>
 
-    word32 cpuid_flags;
-    int cpuid_flags_set = 0;
+    static cpuid_flags_t cpuid_flags = WC_CPUID_INITIALIZER;
 #ifdef WC_C_DYNAMIC_FALLBACK
     #define SHA3_BLOCK (sha3->sha3_block)
     #define SHA3_BLOCK_N (sha3->sha3_block_n)
@@ -298,7 +301,7 @@ void BlockSha3(word64* s)
  */
 #define ROTL64(a, n)    (((a)<<(n))|((a)>>(64-(n))))
 
-#if !defined(STM32_HASH_SHA3)
+#if !defined(STM32_HASH_SHA3) && !defined(PSOC6_HASH_SHA3)
 /* An array of values to XOR for block operation. */
 static const word64 hash_keccak_r[24] =
 {
@@ -533,7 +536,7 @@ do {                                                      \
 while (0)
 #endif /* SHA3_BY_SPEC */
 
-#if !defined(STM32_HASH_SHA3)
+#if !defined(STM32_HASH_SHA3) && !defined(PSOC6_HASH_SHA3)
 /* The block operation performed on the state.
  *
  * s  The state.
@@ -563,7 +566,8 @@ void BlockSha3(word64* s)
 #endif /* STM32_HASH_SHA3 */
 #endif /* !WOLFSSL_ARMASM && !WOLFSSL_RISCV_ASM */
 
-#if !defined(STM32_HASH_SHA3)
+#if !defined(STM32_HASH_SHA3) && !defined(PSOC6_HASH_SHA3)
+#if defined(BIG_ENDIAN_ORDER)
 static WC_INLINE word64 Load64Unaligned(const unsigned char *a)
 {
     return ((word64)a[0] <<  0) |
@@ -583,7 +587,6 @@ static WC_INLINE word64 Load64Unaligned(const unsigned char *a)
  */
 static word64 Load64BitBigEndian(const byte* a)
 {
-#if defined(BIG_ENDIAN_ORDER) || (WOLFSSL_GENERAL_ALIGNMENT == 1)
     word64 n = 0;
     int i;
 
@@ -591,26 +594,8 @@ static word64 Load64BitBigEndian(const byte* a)
         n |= (word64)a[i] << (8 * i);
 
     return n;
-#elif ((WOLFSSL_GENERAL_ALIGNMENT > 0) && (WOLFSSL_GENERAL_ALIGNMENT == 4))
-    word64 n;
-
-    n  =          *(word32*) a;
-    n |= ((word64)*(word32*)(a + 4)) << 32;
-
-    return n;
-#elif ((WOLFSSL_GENERAL_ALIGNMENT > 0) && (WOLFSSL_GENERAL_ALIGNMENT == 2))
-    word64 n;
-
-    n  =          *(word16*) a;
-    n |= ((word64)*(word16*)(a + 2)) << 16;
-    n |= ((word64)*(word16*)(a + 4)) << 32;
-    n |= ((word64)*(word16*)(a + 6)) << 48;
-
-    return n;
-#else
-    return *(const word64*)a;
-#endif
 }
+#endif
 
 /* Initialize the state for a SHA3-224 hash operation.
  *
@@ -630,37 +615,41 @@ static int InitSha3(wc_Sha3* sha3)
 #endif
 
 #ifdef USE_INTEL_SPEEDUP
-    if (!cpuid_flags_set) {
-        cpuid_flags = cpuid_get_flags();
-        cpuid_flags_set = 1;
-#ifdef WC_C_DYNAMIC_FALLBACK
-    }
     {
+        int cpuid_flags_were_updated = cpuid_get_flags_ex(&cpuid_flags);
+#ifdef WC_C_DYNAMIC_FALLBACK
+        (void)cpuid_flags_were_updated;
         if (! CAN_SAVE_VECTOR_REGISTERS()) {
             SHA3_BLOCK = BlockSha3;
             SHA3_BLOCK_N = NULL;
         }
         else
+#else
+        if ((! cpuid_flags_were_updated) && (SHA3_BLOCK != NULL)) {
+        }
+        else
 #endif
-        if (IS_INTEL_BMI1(cpuid_flags) && IS_INTEL_BMI2(cpuid_flags)) {
+        if (IS_INTEL_AVX2(cpuid_flags)) {
+            SHA3_BLOCK = sha3_block_avx2;
+            SHA3_BLOCK_N = sha3_block_n_avx2;
+        }
+        else if (IS_INTEL_BMI1(cpuid_flags) && IS_INTEL_BMI2(cpuid_flags)) {
             SHA3_BLOCK = sha3_block_bmi2;
             SHA3_BLOCK_N = sha3_block_n_bmi2;
         }
-        else if (IS_INTEL_AVX2(cpuid_flags)) {
-            SHA3_BLOCK = sha3_block_avx2;
-            SHA3_BLOCK_N = NULL;
-        }
         else {
             SHA3_BLOCK = BlockSha3;
             SHA3_BLOCK_N = NULL;
         }
     }
 #define SHA3_FUNC_PTR
-#endif
+#endif /* USE_INTEL_SPEEDUP */
 #if defined(__aarch64__) && defined(WOLFSSL_ARMASM)
-    if (!cpuid_flags_set) {
-        cpuid_flags = cpuid_get_flags();
-        cpuid_flags_set = 1;
+    {
+        int cpuid_flags_were_updated = cpuid_get_flags_ex(&cpuid_flags);
+        if ((! cpuid_flags_were_updated) && (SHA3_BLOCK != NULL)) {
+        }
+        else
     #ifdef WOLFSSL_ARMASM_CRYPTO_SHA3
         if (IS_AARCH64_SHA3(cpuid_flags)) {
             SHA3_BLOCK = BlockSha3_crypto;
@@ -699,9 +688,10 @@ static int Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
     word32 i;
     word32 blocks;
 
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
-    if (SHA3_BLOCK == sha3_block_avx2)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
+    if (SHA3_BLOCK == sha3_block_avx2) {
         SAVE_VECTOR_REGISTERS(return _svr_ret;);
+    }
 #endif
     if (sha3->i > 0) {
         byte *t;
@@ -719,9 +709,13 @@ static int Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
         sha3->i = (byte)(sha3->i + i);
 
         if (sha3->i == p * 8) {
+        #if !defined(BIG_ENDIAN_ORDER)
+            xorbuf(sha3->s, sha3->t, (word32)(p * 8));
+        #else
             for (i = 0; i < p; i++) {
                 sha3->s[i] ^= Load64BitBigEndian(sha3->t + 8 * i);
             }
+        #endif
         #ifdef SHA3_FUNC_PTR
             (*SHA3_BLOCK)(sha3->s);
         #else
@@ -740,9 +734,13 @@ static int Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
     }
     #endif
     for (; blocks > 0; blocks--) {
+    #if !defined(BIG_ENDIAN_ORDER)
+        xorbuf(sha3->s, data, (word32)(p * 8));
+    #else
         for (i = 0; i < p; i++) {
             sha3->s[i] ^= Load64Unaligned(data + 8 * i);
         }
+    #endif
     #ifdef SHA3_FUNC_PTR
         (*SHA3_BLOCK)(sha3->s);
     #else
@@ -751,9 +749,10 @@ static int Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
         len -= p * 8U;
         data += p * 8U;
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
-    if (SHA3_BLOCK == sha3_block_avx2)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
+    if (SHA3_BLOCK == sha3_block_avx2) {
         RESTORE_VECTOR_REGISTERS();
+    }
 #endif
     if (len > 0) {
         XMEMCPY(sha3->t, data, len);
@@ -775,12 +774,25 @@ static int Sha3Final(wc_Sha3* sha3, byte padChar, byte* hash, byte p, word32 l)
 {
     word32 rate = p * 8U;
     word32 j;
+#if defined(BIG_ENDIAN_ORDER)
     word32 i;
+#endif
 
+#if !defined(BIG_ENDIAN_ORDER)
+    xorbuf(sha3->s, sha3->t, sha3->i);
+#ifdef WOLFSSL_HASH_FLAGS
+    if ((p == WC_SHA3_256_COUNT) && (sha3->flags & WC_HASH_SHA3_KECCAK256)) {
+        padChar = 0x01;
+    }
+#endif
+    ((byte*)sha3->s)[sha3->i ] ^= padChar;
+    ((byte*)sha3->s)[rate - 1] ^= 0x80;
+#else
     sha3->t[rate - 1]  = 0x00;
 #ifdef WOLFSSL_HASH_FLAGS
-    if ((p == WC_SHA3_256_COUNT) && (sha3->flags & WC_HASH_SHA3_KECCAK256))
+    if ((p == WC_SHA3_256_COUNT) && (sha3->flags & WC_HASH_SHA3_KECCAK256)) {
         padChar = 0x01;
+    }
 #endif
     sha3->t[sha3->i ]  = padChar;
     sha3->t[rate - 1] |= 0x80;
@@ -790,8 +802,9 @@ static int Sha3Final(wc_Sha3* sha3, byte padChar, byte* hash, byte p, word32 l)
     for (i = 0; i < p; i++) {
         sha3->s[i] ^= Load64BitBigEndian(sha3->t + 8 * i);
     }
+#endif
 
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
     if (SHA3_BLOCK == sha3_block_avx2)
         SAVE_VECTOR_REGISTERS(return _svr_ret;);
 #endif
@@ -819,9 +832,10 @@ static int Sha3Final(wc_Sha3* sha3, byte padChar, byte* hash, byte p, word32 l)
     #endif
         XMEMCPY(hash + j, sha3->s, l - j);
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
-    if (SHA3_BLOCK == sha3_block_avx2)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
+    if (SHA3_BLOCK == sha3_block_avx2) {
         RESTORE_VECTOR_REGISTERS();
+    }
 #endif
 
     return 0;
@@ -903,6 +917,78 @@ static int Sha3Final(wc_Sha3* sha3, byte padChar, byte* hash, byte p, word32 l)
 
         return ret;
     }
+#elif defined(PSOC6_HASH_SHA3)
+
+static int wc_InitSha3(wc_Sha3* sha3, void* heap, int devId)
+{
+    int ret;
+    if (sha3 == NULL) {
+        return BAD_FUNC_ARG;
+    }
+    (void)devId;
+    (void)heap;
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Initialize hash state for SHA-3 operation */
+        ret = wc_Psoc6_Sha3_Init(sha3);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+static int wc_Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
+{
+    int ret;
+
+    if (sha3 == NULL || (data == NULL && len > 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    if (data == NULL && len == 0) {
+        /* valid, but do nothing */
+        return 0;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Perform SHA3 on the input data and update the hash state */
+        ret = wc_Psoc6_Sha3_Update(sha3, data, len, p);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+static int wc_Sha3Final(wc_Sha3* sha3, byte* hash, byte p, byte len)
+{
+    int ret;
+
+    if (sha3 == NULL || hash == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Finalize SHA3 operations and produce digest */
+        ret = wc_Psoc6_Sha3_Final(sha3, 0x06, hash, p, len);
+        if (ret == 0) {
+            /* Initialize hash state for SHA-3 operation */
+            ret = wc_Psoc6_Sha3_Init(sha3);
+        }
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
 #else
 
 /* Initialize the state for a SHA-3 hash operation.
@@ -930,6 +1016,9 @@ static int wc_InitSha3(wc_Sha3* sha3, void* heap, int devId)
 #endif
 #if defined(WOLF_CRYPTO_CB)
     sha3->devId = devId;
+    /* Set to none to determine the hash type later */
+    /* in the update/final functions based on the p value */
+    sha3->hashType = WC_HASH_TYPE_NONE;
 #endif
     (void)devId;
 
@@ -962,15 +1051,22 @@ static int wc_Sha3Update(wc_Sha3* sha3, const byte* data, word32 len, byte p)
     if (sha3->devId != INVALID_DEVID)
     #endif
     {
-        int hash_type = WC_HASH_TYPE_NONE;
-        switch (p) {
-            case WC_SHA3_224_COUNT: hash_type = WC_HASH_TYPE_SHA3_224; break;
-            case WC_SHA3_256_COUNT: hash_type = WC_HASH_TYPE_SHA3_256; break;
-            case WC_SHA3_384_COUNT: hash_type = WC_HASH_TYPE_SHA3_384; break;
-            case WC_SHA3_512_COUNT: hash_type = WC_HASH_TYPE_SHA3_512; break;
-            default: return BAD_FUNC_ARG;
+        /* If the hash type is not set, determine it based on the p value */
+        /* We can skip the switch statement if the hash type set already */
+        if (sha3->hashType == WC_HASH_TYPE_NONE) {
+            switch (p) {
+                case WC_SHA3_224_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_224; break;
+                case WC_SHA3_256_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_256; break;
+                case WC_SHA3_384_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_384; break;
+                case WC_SHA3_512_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_512; break;
+                default: return BAD_FUNC_ARG;
+            }
         }
-        ret = wc_CryptoCb_Sha3Hash(sha3, hash_type, data, len, NULL);
+        ret = wc_CryptoCb_Sha3Hash(sha3, sha3->hashType, data, len, NULL);
         if (ret != WC_NO_ERR_TRACE(CRYPTOCB_UNAVAILABLE))
             return ret;
         /* fall-through when unavailable */
@@ -1016,15 +1112,22 @@ static int wc_Sha3Final(wc_Sha3* sha3, byte* hash, byte p, byte len)
     if (sha3->devId != INVALID_DEVID)
     #endif
     {
-        int hash_type = WC_HASH_TYPE_NONE;
-        switch (p) {
-            case WC_SHA3_224_COUNT: hash_type = WC_HASH_TYPE_SHA3_224; break;
-            case WC_SHA3_256_COUNT: hash_type = WC_HASH_TYPE_SHA3_256; break;
-            case WC_SHA3_384_COUNT: hash_type = WC_HASH_TYPE_SHA3_384; break;
-            case WC_SHA3_512_COUNT: hash_type = WC_HASH_TYPE_SHA3_512; break;
-            default: return BAD_FUNC_ARG;
+        /* If the hash type is not set, determine it based on the p value */
+        /* We can skip the switch statement if the hash type is set already */
+        if (sha3->hashType == WC_HASH_TYPE_NONE) {
+            switch (p) {
+                case WC_SHA3_224_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_224; break;
+                case WC_SHA3_256_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_256; break;
+                case WC_SHA3_384_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_384; break;
+                case WC_SHA3_512_COUNT:
+                    sha3->hashType = WC_HASH_TYPE_SHA3_512; break;
+                default: return BAD_FUNC_ARG;
+            }
         }
-        ret = wc_CryptoCb_Sha3Hash(sha3, hash_type, NULL, 0, hash);
+        ret = wc_CryptoCb_Sha3Hash(sha3, sha3->hashType, NULL, 0, hash);
         if (ret != WC_NO_ERR_TRACE(CRYPTOCB_UNAVAILABLE))
             return ret;
         /* fall-through when unavailable */
@@ -1052,6 +1155,7 @@ static int wc_Sha3Final(wc_Sha3* sha3, byte* hash, byte p, byte len)
     return InitSha3(sha3);  /* reset state */
 }
 #endif
+
 /* Dispose of any dynamically allocated data from the SHA3-384 operation.
  * (Required for async ops.)
  *
@@ -1060,16 +1164,46 @@ static int wc_Sha3Final(wc_Sha3* sha3, byte* hash, byte p, byte len)
  */
 static void wc_Sha3Free(wc_Sha3* sha3)
 {
+#if defined(WOLF_CRYPTO_CB) && defined(WOLF_CRYPTO_CB_FREE)
+    int ret = 0;
+#endif
+
     (void)sha3;
 
+#if defined(WOLF_CRYPTO_CB) && defined(WOLF_CRYPTO_CB_FREE)
+    if (sha3 == NULL)
+        return;
+
+    #ifndef WOLF_CRYPTO_CB_FIND
+    if (sha3->devId != INVALID_DEVID)
+    #endif
+    {
+        ret = wc_CryptoCb_Free(sha3->devId, WC_ALGO_TYPE_HASH,
+                         sha3->hashType, (void*)sha3);
+        /* If they want the standard free, they can call it themselves */
+        /* via their callback setting devId to INVALID_DEVID */
+        /* otherwise assume the callback handled it */
+        if (ret != WC_NO_ERR_TRACE(CRYPTOCB_UNAVAILABLE))
+            return;
+        /* fall-through when unavailable */
+    }
+
+    /* silence compiler warning */
+    (void)ret;
+
+#endif /* WOLF_CRYPTO_CB && WOLF_CRYPTO_CB_FREE */
+
 #if defined(WOLFSSL_ASYNC_CRYPT) && defined(WC_ASYNC_ENABLE_SHA3)
     if (sha3 == NULL)
         return;
 
     wolfAsync_DevCtxFree(&sha3->asyncDev, WOLFSSL_ASYNC_MARKER_SHA3);
 #endif /* WOLFSSL_ASYNC_CRYPT */
-}
 
+#if defined(PSOC6_HASH_SHA3)
+    wc_Psoc6_Sha_Free();
+#endif
+}
 
 /* Copy the state of the SHA3 operation.
  *
@@ -1084,11 +1218,32 @@ static int wc_Sha3Copy(wc_Sha3* src, wc_Sha3* dst)
     if (src == NULL || dst == NULL)
         return BAD_FUNC_ARG;
 
+#if defined(WOLF_CRYPTO_CB) && defined(WOLF_CRYPTO_CB_COPY)
+    #ifndef WOLF_CRYPTO_CB_FIND
+    if (src->devId != INVALID_DEVID)
+    #endif
+    {
+        /* Cast the source and destination to be void to keep the abstraction */
+        ret = wc_CryptoCb_Copy(src->devId, WC_ALGO_TYPE_HASH,
+                               src->hashType, (void*)src, (void*)dst);
+        if (ret != WC_NO_ERR_TRACE(CRYPTOCB_UNAVAILABLE))
+            return ret;
+        /* fall-through when unavailable */
+    }
+    ret = 0; /* Reset ret to 0 to avoid returning the callback error code */
+#endif /* WOLF_CRYPTO_CB && WOLF_CRYPTO_CB_COPY */
+
     XMEMCPY(dst, src, sizeof(wc_Sha3));
 
 #if defined(WOLFSSL_ASYNC_CRYPT) && defined(WC_ASYNC_ENABLE_SHA3)
     ret = wolfAsync_DevCopy(&src->asyncDev, &dst->asyncDev);
 #endif
+
+#if defined(PSOC6_HASH_SHA3)
+    /* Re-initialize internal pointers in hash_state that point inside sha_buffers */
+    dst->hash_state.hash = (uint8_t*)((cy_stc_crypto_v2_sha3_buffers_t *)&dst->sha_buffers)->hash;
+#endif
+
 #ifdef WOLFSSL_HASH_FLAGS
      dst->flags |= WC_HASH_FLAG_ISCOPY;
 #endif
@@ -1121,7 +1276,6 @@ static int wc_Sha3GetHash(wc_Sha3* sha3, byte* hash, byte p, byte len)
     return ret;
 }
 
-
 /* Initialize the state for a SHA3-224 hash operation.
  *
  * sha3   wc_Sha3 object holding state.
@@ -1439,6 +1593,101 @@ int wc_InitShake128(wc_Shake* shake, void* heap, int devId)
     return wc_InitSha3(shake, heap, devId);
 }
 
+#if defined(PSOC6_HASH_SHA3)
+
+int wc_Shake128_Update(wc_Shake* shake, const byte* data, word32 len)
+{
+    int ret;
+    if (shake == NULL || (data == NULL && len > 0)) {
+         return BAD_FUNC_ARG;
+    }
+
+    if (data == NULL && len == 0) {
+        /* valid, but do nothing */
+        return 0;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Perform SHA3 on the input data and update the hash state */
+        ret = wc_Psoc6_Sha3_Update(shake, data, len, WC_SHA3_128_COUNT);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+int wc_Shake128_Final(wc_Shake* shake, byte* hash, word32 hashLen)
+{
+    int ret;
+
+    if (shake == NULL || hash == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Finalize SHA3 operations and produce digest */
+        ret = wc_Psoc6_Sha3_Final(shake, 0x1f, hash, WC_SHA3_128_COUNT, hashLen);
+        if (ret == 0) {
+            /* Initialize hash state for SHA-3 operation */
+            ret = wc_Psoc6_Sha3_Init(shake);
+        }
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+
+}
+
+int wc_Shake128_Absorb(wc_Shake* shake, const byte* data, word32 len)
+{
+    int ret;
+
+    if ((shake == NULL) || (data == NULL && len != 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Perform SHA3 on the input data and update the hash state */
+        ret = wc_Psoc6_Sha3_Update(shake, data, len, WC_SHA3_128_COUNT);
+        if (ret == 0) {
+            /* Finalize SHA3 operations and produce digest */
+            ret = wc_Psoc6_Sha3_Final(shake, 0x1f, NULL, WC_SHA3_128_COUNT, 0);
+        }
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+
+int wc_Shake128_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
+{
+    int ret;
+    if ((shake == NULL) || (out == NULL && blockCnt != 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Squeeze output blocks from current hash state */
+        ret = wc_Psoc6_Shake_SqueezeBlocks(shake, out, blockCnt);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+#else
 /* Update the SHAKE128 hash state with message data.
  *
  * shake  wc_Shake object holding state.
@@ -1529,7 +1778,7 @@ int wc_Shake128_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
     if ((shake == NULL) || (out == NULL && blockCnt != 0)) {
         return BAD_FUNC_ARG;
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
     if (SHA3_BLOCK == sha3_block_avx2)
         SAVE_VECTOR_REGISTERS(return _svr_ret;);
 #endif
@@ -1546,13 +1795,15 @@ int wc_Shake128_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
     #endif
         out += WC_SHA3_128_COUNT * 8;
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
     if (SHA3_BLOCK == sha3_block_avx2)
         RESTORE_VECTOR_REGISTERS();
 #endif
 
     return 0;
 }
+#endif
+
 
 /* Dispose of any dynamically allocated data from the SHAKE128 operation.
  * (Required for async ops.)
@@ -1590,6 +1841,100 @@ int wc_InitShake256(wc_Shake* shake, void* heap, int devId)
     return wc_InitSha3(shake, heap, devId);
 }
 
+
+#ifdef PSOC6_HASH_SHA3
+
+int wc_Shake256_Update(wc_Shake* shake, const byte* data, word32 len)
+{
+    int ret;
+    if (shake == NULL || (data == NULL && len > 0)) {
+         return BAD_FUNC_ARG;
+    }
+
+    if (data == NULL && len == 0) {
+        /* valid, but do nothing */
+        return 0;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Perform SHA3 on the input data and update the hash state */
+        ret = wc_Psoc6_Sha3_Update(shake, data, len, WC_SHA3_256_COUNT);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+int wc_Shake256_Final(wc_Shake* shake, byte* hash, word32 hashLen)
+{
+    int ret;
+    if (shake == NULL || hash == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Finalize SHA3 operations and produce digest */
+        ret = wc_Psoc6_Sha3_Final(shake, 0x1f, hash, WC_SHA3_256_COUNT, hashLen);
+        if (ret == 0) {
+            /* Initialize hash state for SHA-3 operation */
+            ret = wc_Psoc6_Sha3_Init(shake);
+        }
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+int wc_Shake256_Absorb(wc_Shake* shake, const byte* data, word32 len)
+{
+    int ret;
+
+    if ((shake == NULL) || (data == NULL && len != 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Perform SHA3 on the input data and update the hash state */
+        ret = wc_Psoc6_Sha3_Update(shake, data, len, WC_SHA3_256_COUNT);
+        if (ret == 0) {
+            /* Finalize SHA3 operations and produce digest */
+            ret = wc_Psoc6_Sha3_Final(shake, 0x1f, NULL, WC_SHA3_256_COUNT, 0);
+        }
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+int wc_Shake256_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
+{
+    int ret;
+    if ((shake == NULL) || (out == NULL && blockCnt != 0)) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* Lock the mutex to perform crypto operations */
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+        /* Squeeze output blocks from current hash state */
+        ret = wc_Psoc6_Shake_SqueezeBlocks(shake, out, blockCnt);
+        /* Release the lock */
+        wolfSSL_CryptHwMutexUnLock();
+    }
+
+    return ret;
+}
+
+#else
 /* Update the SHAKE256 hash state with message data.
  *
  * shake  wc_Shake object holding state.
@@ -1674,7 +2019,7 @@ int wc_Shake256_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
     if ((shake == NULL) || (out == NULL && blockCnt != 0)) {
         return BAD_FUNC_ARG;
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
     if (SHA3_BLOCK == sha3_block_avx2)
         SAVE_VECTOR_REGISTERS(return _svr_ret;);
 #endif
@@ -1691,13 +2036,14 @@ int wc_Shake256_SqueezeBlocks(wc_Shake* shake, byte* out, word32 blockCnt)
     #endif
         out += WC_SHA3_256_COUNT * 8;
     }
-#if defined(WOLFSSL_LINUXKM) && defined(USE_INTEL_SPEEDUP)
+#if defined(WOLFSSL_USE_SAVE_VECTOR_REGISTERS) && defined(USE_INTEL_SPEEDUP)
     if (SHA3_BLOCK == sha3_block_avx2)
         RESTORE_VECTOR_REGISTERS();
 #endif
 
     return 0;
 }
+#endif
 
 /* Dispose of any dynamically allocated data from the SHAKE256 operation.
  * (Required for async ops.)
