@@ -1,12 +1,12 @@
 /* wolfevent.c
  *
- * Copyright (C) 2006-2017 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -19,18 +19,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
-#ifdef HAVE_CONFIG_H
-    #include <config.h>
-#endif
-
-#include <wolfssl/wolfcrypt/settings.h>
-
+#include <wolfssl/wolfcrypt/libwolfssl_sources.h>
 
 #ifdef HAVE_WOLF_EVENT
 
 #include <wolfssl/internal.h>
 #include <wolfssl/error-ssl.h>
-#include <wolfssl/wolfcrypt/error-crypt.h>
 
 #include <wolfssl/wolfcrypt/wolfevent.h>
 
@@ -55,7 +49,7 @@ int wolfEvent_Init(WOLF_EVENT* event, WOLF_EVENT_TYPE type, void* context)
 
 int wolfEvent_Poll(WOLF_EVENT* event, WOLF_EVENT_FLAG flags)
 {
-    int ret = BAD_COND_E;
+    int ret = WC_NO_ERR_TRACE(BAD_COND_E);
 
     /* Check hardware */
 #ifdef WOLFSSL_ASYNC_CRYPT
@@ -180,8 +174,12 @@ int wolfEventQueue_Remove(WOLF_EVENT_QUEUE* queue, WOLF_EVENT* event)
     else {
         WOLF_EVENT* next = event->next;
         WOLF_EVENT* prev = event->prev;
-        next->prev = prev;
-        prev->next = next;
+        if ((next == NULL) || (prev == NULL)) {
+            ret = BAD_STATE_E;
+        } else {
+            next->prev = prev;
+            prev->next = next;
+        }
     }
     queue->count--;
 
@@ -205,7 +203,7 @@ int wolfEventQueue_Poll(WOLF_EVENT_QUEUE* queue, void* context_filter,
     }
 #endif
 
-    /* itterate event queue */
+    /* iterate event queue */
     for (event = queue->head; event != NULL; event = event->next)
     {
         /* optional filter based on context */
