@@ -1,12 +1,12 @@
 /* des.h
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -49,8 +49,8 @@ typedef unsigned int WOLFSSL_DES_LONG;
 
 
 enum {
-    DES_ENCRYPT = 1,
-    DES_DECRYPT = 0
+    WC_DES_ENCRYPT = 1,
+    WC_DES_DECRYPT = 0
 };
 
 
@@ -87,6 +87,13 @@ WOLFSSL_API void wolfSSL_DES_ecb_encrypt(WOLFSSL_DES_cblock* desa,
 WOLFSSL_API int wolfSSL_DES_check_key_parity(WOLFSSL_DES_cblock *myDes);
 
 
+#ifndef OPENSSL_COEXIST
+
+enum {
+    DES_ENCRYPT = WC_DES_ENCRYPT,
+    DES_DECRYPT = WC_DES_DECRYPT
+};
+
 typedef WOLFSSL_DES_cblock DES_cblock;
 typedef WOLFSSL_const_DES_cblock const_DES_cblock;
 typedef WOLFSSL_DES_key_schedule DES_key_schedule;
@@ -106,6 +113,8 @@ typedef WOLFSSL_DES_LONG DES_LONG;
 #define DES_cbc_cksum         wolfSSL_DES_cbc_cksum
 #define DES_check_key_parity  wolfSSL_DES_check_key_parity
 
+#endif /* !OPENSSL_COEXIST */
+
 #ifdef __cplusplus
     } /* extern "C" */
 #endif
