@@ -1,12 +1,12 @@
 /* kdf.h
  *
- * Copyright (C) 2006-2023 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -35,10 +35,19 @@
 
 #include <wolfssl/wolfcrypt/hmac.h>
 
+#ifdef HAVE_CMAC_KDF
+    #include <wolfssl/wolfcrypt/cmac.h>
+#endif /* HAVE_CMAC_KDF */
+
 #ifdef __cplusplus
     extern "C" {
 #endif
 
+#if FIPS_VERSION3_GE(6,0,0)
+    extern const unsigned int wolfCrypt_FIPS_kdf_ro_sanity[2];
+    WOLFSSL_LOCAL int wolfCrypt_FIPS_KDF_sanity(void);
+#endif
+
 enum max_prf {
 #ifdef HAVE_FFDHE_8192
     MAX_PRF_HALF        = 516, /* Maximum half secret len */
@@ -76,9 +85,20 @@ enum {
     MAX_TLS13_HKDF_LABEL_SZ = 47 + WC_MAX_DIGEST_SIZE
 };
 
+WOLFSSL_API int wc_Tls13_HKDF_Extract_ex(byte* prk, const byte* salt,
+                          word32 saltLen, byte* ikm, word32 ikmLen, int digest,
+                          void* heap, int devId);
+
 WOLFSSL_API int wc_Tls13_HKDF_Extract(byte* prk, const byte* salt,
                           word32 saltLen, byte* ikm, word32 ikmLen, int digest);
 
+WOLFSSL_API int wc_Tls13_HKDF_Expand_Label_ex(byte* okm, word32 okmLen,
+                             const byte* prk, word32 prkLen,
+                             const byte* protocol, word32 protocolLen,
+                             const byte* label, word32 labelLen,
+                             const byte* info, word32 infoLen,
+                             int digest, void* heap, int devId);
+
 WOLFSSL_API int wc_Tls13_HKDF_Expand_Label(byte* okm, word32 okmLen,
                              const byte* prk, word32 prkLen,
                              const byte* protocol, word32 protocolLen,
@@ -121,26 +141,54 @@ WOLFSSL_API int wc_SSH_KDF(byte hashId, byte keyId,
 /* Length of index for SRTCP KDF. */
 #define WC_SRTCP_INDEX_LEN              4
 
+/* Indicators */
+enum {
+    WC_SRTCP_32BIT_IDX = 0,
+    WC_SRTCP_48BIT_IDX = 1
+};
+
 /* Maximum length of salt that can be used with SRTP/SRTCP. */
 #define WC_SRTP_MAX_SALT    14
 
 WOLFSSL_API int wc_SRTP_KDF(const byte* key, word32 keySz, const byte* salt,
-    word32 saltSz, int kdrIdx, const byte* index, byte* key1, word32 key1Sz,
+    word32 saltSz, int kdrIdx, const byte* idx, byte* key1, word32 key1Sz,
     byte* key2, word32 key2Sz, byte* key3, word32 key3Sz);
 WOLFSSL_API int wc_SRTCP_KDF(const byte* key, word32 keySz, const byte* salt,
-    word32 saltSz, int kdrIdx, const byte* index, byte* key1, word32 key1Sz,
+    word32 saltSz, int kdrIdx, const byte* idx, byte* key1, word32 key1Sz,
     byte* key2, word32 key2Sz, byte* key3, word32 key3Sz);
+WOLFSSL_API int wc_SRTCP_KDF_ex(const byte* key, word32 keySz, const byte* salt,
+    word32 saltSz, int kdrIdx, const byte* idx, byte* key1, word32 key1Sz,
+    byte* key2, word32 key2Sz, byte* key3, word32 key3Sz, int idxLenIndicator);
 WOLFSSL_API int wc_SRTP_KDF_label(const byte* key, word32 keySz,
-    const byte* salt, word32 saltSz, int kdrIdx, const byte* index, byte label,
+    const byte* salt, word32 saltSz, int kdrIdx, const byte* idx, byte label,
     byte* outKey, word32 outKeySz);
 WOLFSSL_API int wc_SRTCP_KDF_label(const byte* key, word32 keySz,
-    const byte* salt, word32 saltSz, int kdrIdx, const byte* index, byte label,
+    const byte* salt, word32 saltSz, int kdrIdx, const byte* idx, byte label,
     byte* outKey, word32 outKeySz);
 
 WOLFSSL_API int wc_SRTP_KDF_kdr_to_idx(word32 kdr);
 
 #endif /* WC_SRTP_KDF */
 
+#ifdef WC_KDF_NIST_SP_800_56C
+WOLFSSL_API int wc_KDA_KDF_onestep(const byte* z, word32 zSz,
+    const byte* fixedInfo, word32 fixedInfoSz, word32 derivedSecretSz,
+    enum wc_HashType hashType, byte* output, word32 outputSz);
+#endif
+
+#ifdef HAVE_CMAC_KDF
+WOLFSSL_API int wc_KDA_KDF_twostep_cmac(const byte * salt, word32 salt_len,
+                                        const byte* z, word32 zSz,
+                                        const byte* fixedInfo,
+                                        word32 fixedInfoSz,
+                                        byte* output, word32 outputSz,
+                                        void* heap, int devId);
+WOLFSSL_API int wc_KDA_KDF_PRF_cmac(const byte* Kin, word32 KinSz,
+                                    const byte* fixedInfo, word32 fixedInfoSz,
+                                    byte* Kout, word32 KoutSz, CmacType type,
+                                    void* heap, int devId);
+#endif /* HAVE_CMAC_KDF */
+
 #ifdef __cplusplus
     } /* extern "C" */
 #endif
